@@ -1,6 +1,5 @@
 /*
- *  Copyright (c) 2014, Peter Haag
- *  Copyright (c) 2009, Peter Haag
+ *  Copyright (c) 2009-2020, Peter Haag
  *  Copyright (c) 2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *  
@@ -28,15 +27,11 @@
  *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
  *  POSSIBILITY OF SUCH DAMAGE.
  *  
- *  $Author: haag $
- *
- *  $Id: collector.c 69 2010-09-09 07:17:43Z haag $
- *
- *  $LastChangedRevision: 69 $
- *	
  */
 
-#include "config.h"
+#ifdef HAVE_STDINT_H
+#include <stdint.h>
+#endif
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -50,7 +45,6 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <unistd.h>
-#include <syslog.h>
 #include <stdarg.h>
 
 #include <time.h>
@@ -59,26 +53,15 @@
 #include <stdint.h>
 #endif
 
-#ifndef DEVEL
-#   define dbg_printf(...) /* printf(__VA_ARGS__) */
-#else
-#   define dbg_printf(...) printf(__VA_ARGS__)
-#endif
-
 #include "util.h"
-#include "nf_common.h"
+#include "nfdump.h"
 #include "nffile.h"
-#include "nfxstat.h"
 #include "bookkeeper.h"
 #include "collector.h"
 #include "nfx.h"
 
 #include "nffile_inline.c"
 
-/* globals */
-uint32_t default_sampling   = 1;
-uint32_t overwrite_sampling = 0;
-
 /* local variables */
 static uint32_t	exporter_sysid = 0;
 static char *DynamicSourcesDir = NULL;
@@ -139,7 +122,6 @@ int ok;
 	(*source)->bookkeeper 	  	  = NULL;
 	(*source)->any_source 	  	  = 0;
 	(*source)->exporter_data  	  = NULL;
-	(*source)->xstat 		  	  = NULL;
 	(*FlowSource)->exporter_count = 0;
 
 	// separate IP address from ident
@@ -160,15 +142,15 @@ int ok;
 		uint64_t _ip[2];
 		ok = inet_pton(PF_INET6, p, _ip);
 		(*source)->sa_family = PF_INET6;
-		(*source)->ip.v6[0] = ntohll(_ip[0]);
-		(*source)->ip.v6[1] = ntohll(_ip[1]);
+		(*source)->ip.V6[0] = ntohll(_ip[0]);
+		(*source)->ip.V6[1] = ntohll(_ip[1]);
 	} else {
 		uint32_t _ip;
 		ok = inet_pton(PF_INET, p, &_ip);
 		(*source)->sa_family = PF_INET;
-		(*source)->ip.v6[0] = 0;
-		(*source)->ip.v6[1] = 0;
-		(*source)->ip.v4 = ntohl(_ip);
+		(*source)->ip.V6[0] = 0;
+		(*source)->ip.V6[1] = 0;
+		(*source)->ip.V4 = ntohl(_ip);
 	}
 	switch (ok) {
 		case 0:
@@ -200,22 +182,24 @@ int ok;
 		exit(255);
 	}
 
+	char *path = realpath(q, NULL);
+	if ( !path ) {
+		fprintf(stderr, "realpath() error %s: %s\n", q, strerror(errno));
+		return 0;
+	}
+
 	// check for existing path
-	if ( stat(q, &fstat) ) {
-		fprintf(stderr, "stat() error %s: %s\n", q, strerror(errno));
+	if ( stat(path, &fstat) ) {
+		fprintf(stderr, "stat() error %s: %s\n", path, strerror(errno));
 		return 0;
 	}
 	if ( !(fstat.st_mode & S_IFDIR) ) {
-		fprintf(stderr, "No such directory: %s\n", q);
+		fprintf(stderr, "Not a directory: %s\n", path);
 		return 0;
 	}
 
 	// remember path
-	(*source)->datadir = strdup(q);
-	if ( !(*source)->datadir ) {
-		fprintf(stderr, "strdup() error: %s\n", strerror(errno));
-		return 0;
-	}
+	(*source)->datadir = path;
 
 	// cache current collector file
 	if ( snprintf(s, MAXPATHLEN-1, "%s/%s.%lu", (*source)->datadir , NF_DUMPFILE, (unsigned long)getpid() ) >= (MAXPATHLEN-1)) {
@@ -248,7 +232,6 @@ char s[MAXPATHLEN];
 	(*FlowSource)->bookkeeper 	  = NULL;
 	(*FlowSource)->any_source 	  = 1;
 	(*FlowSource)->exporter_data  = NULL;
-	(*FlowSource)->xstat 	  	  = NULL;
 	(*FlowSource)->exporter_count = 0;
 
 	// fill in ident
@@ -331,7 +314,6 @@ int				err;
 	(*source)->bookkeeper 	  	  = NULL;
 	(*source)->any_source 	  	  = 0;
 	(*source)->exporter_data  	  = NULL;
-	(*source)->xstat 		  	  = NULL;
 	(*FlowSource)->exporter_count = 0;
 
 	switch (ss->ss_family) {
@@ -346,9 +328,9 @@ int				err;
 			}
 #endif
 			(*source)->sa_family = PF_INET;
-			(*source)->ip.v6[0] = 0;
-			(*source)->ip.v6[1] = 0;
-			(*source)->ip.v4 = ntohl(u.sa_in->sin_addr.s_addr);
+			(*source)->ip.V6[0] = 0;
+			(*source)->ip.V6[1] = 0;
+			(*source)->ip.V4 = ntohl(u.sa_in->sin_addr.s_addr);
 			ptr 	   = &u.sa_in->sin_addr;
 			} break;
 		case PF_INET6: {
@@ -365,14 +347,14 @@ int				err;
 #endif
 			// ptr = &((struct sockaddr_in6 *)sa)->sin6_addr;
 			(*source)->sa_family = PF_INET6;
-			(*source)->ip.v6[0] = ntohll(ip_ptr[0]);
-			(*source)->ip.v6[1] = ntohll(ip_ptr[1]);
+			(*source)->ip.V6[0] = ntohll(ip_ptr[0]);
+			(*source)->ip.V6[1] = ntohll(ip_ptr[1]);
 			ptr = &u.sa_in6->sin6_addr;
 			} break;
 		default:
 			// keep compiler happy
-			(*source)->ip.v6[0] = 0;
-			(*source)->ip.v6[1] = 0;
+			(*source)->ip.V6[0] = 0;
+			(*source)->ip.V6[1] = 0;
 			ptr   = NULL;
 
 			LogError("Unknown sa fanily: %d in '%s', line '%d'", ss->ss_family, __FILE__, __LINE__ );
@@ -391,10 +373,6 @@ int				err;
 	ident[99] = '\0';
 	dbg_printf("Dynamic Flow Source IP: %s\n", ident);
 
-	if ( strchr(ident, ':') ) { // condense IPv6 addresses
-		condense_v6(ident);
-	}
-
 	s = ident;
 	while ( *s != '\0' ) {
 		if ( *s == '.' || *s == ':' ) 
@@ -537,14 +515,14 @@ int FlushInfoExporter(FlowSource_t *fs, exporter_info_record_t *exporter) {
 		char ipstr[IP_STRING_LEN];
 		printf("Flush Exporter: ");
 		if ( exporter->sa_family == AF_INET ) {
-			uint32_t _ip = htonl(exporter->ip.v4);
+			uint32_t _ip = htonl(exporter->ip.V4);
 			inet_ntop(AF_INET, &_ip, ipstr, sizeof(ipstr));
 			printf("SysID: %u, IP: %16s, version: %u, ID: %2u\n", exporter->sysid,
 				ipstr, exporter->version, exporter->id);
 		} else if ( exporter->sa_family == AF_INET6 ) {
 			uint64_t _ip[2];
-			_ip[0] = htonll(exporter->ip.v6[0]);
-			_ip[1] = htonll(exporter->ip.v6[1]);
+			_ip[0] = htonll(exporter->ip.V6[0]);
+			_ip[1] = htonll(exporter->ip.V6[1]);
 			inet_ntop(AF_INET6, &_ip, ipstr, sizeof(ipstr));
 			printf("SysID: %u, IP: %40s, version: %u, ID: %2u\n", exporter->sysid,
 				ipstr, exporter->version, exporter->id);
@@ -581,11 +559,11 @@ int FlushInfoSampler(FlowSource_t *fs, sampler_info_record_t *sampler) {
 } // End of FlushInfoSampler
 
 void FlushStdRecords(FlowSource_t *fs) {
-generic_exporter_t *e = fs->exporter_data;
+exporter_t *e = fs->exporter_data;
 int i;
 
 	while ( e ) {
-		generic_sampler_t *sampler = e->sampler;
+		sampler_t *sampler = e->sampler;
 		AppendToBuffer(fs->nffile, (void *)&(e->info), e->info.header.size);
 		while ( sampler ) {
 			AppendToBuffer(fs->nffile, (void *)&(sampler->info), sampler->info.header.size);
@@ -603,7 +581,7 @@ int i;
 } // End of FlushStdRecords
 
 void FlushExporterStats(FlowSource_t *fs) {
-generic_exporter_t *e = fs->exporter_data;
+exporter_t *e = fs->exporter_data;
 exporter_stats_record_t	*exporter_stats;
 uint32_t i, size;
 
@@ -638,12 +616,13 @@ uint32_t i, size;
 		exporter_stats->stat[i].packets 		 = e->packets;
 		exporter_stats->stat[i].flows 			 = e->flows;
 #ifdef DEVEL
-		printf("Stat: SysID: %u, version: %u, ID: %2u, Packets: %llu, Flows: %llu, Sequence Failures: %u\n", e->info.sysid,
-			e->info.version, e->info.id, e->packets, e->flows, e->sequence_failure);
+		printf("Stat: SysID: %u, version: %u, ID: %2u, Packets: %llu, Flows: %llu, Sequence Failures: %u, Padding Errors: %u\n", 
+			e->info.sysid, e->info.version, e->info.id, e->packets, e->flows, e->sequence_failure, e->padding_errors);
 
 #endif
 		// reset counters
 		e->sequence_failure = 0;
+		e->padding_errors   = 0;
 		e->packets 			= 0;
 		e->flows 			= 0;
 
@@ -660,17 +639,3 @@ uint32_t i, size;
  
 } // End of FlushExporterStats
 
-
-
-int HasOptionTable(FlowSource_t *fs, uint16_t id ) {
-option_offset_t *t;
-
-	t = fs->option_offset_table;
-	while ( t && t->id != id )
-		t = t->next;
-
-	dbg_printf("Has option table: %s\n", t == NULL ? "not found" : "found");
-
-	return t != NULL;
-
-} // End of HasOptionTable
