@@ -1,6 +1,5 @@
 /*
- *  Copyright (c) 2014, Peter Haag
- *  Copyright (c) 2009, Peter Haag
+ *  Copyright (c) 2009-2019, Peter Haag
  *  Copyright (c) 2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *  
@@ -28,69 +27,30 @@
  *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
  *  POSSIBILITY OF SUCH DAMAGE.
  *  
- *  $Author: haag $
- *
- *  $Id: collector.h 51 2010-01-29 09:01:54Z haag $
- *
- *  $LastChangedRevision: 51 $
- *	
  */
 
 #ifndef _COLLECTOR_H
 #define _COLLECTOR_H 1
 
+#include "config.h"
+
+#include <sys/types.h>
+#ifdef HAVE_STDINT_H
+#include <stdint.h>
+#endif
+#include <sys/socket.h>
+
+#include "exporter.h"
+#include "bookkeeper.h"
+#include "nffile.h"
+
 #define FNAME_SIZE  256
-#define IDENT_SIZE  32
 
-typedef struct srecord_s {
-    char    fname[FNAME_SIZE];      // file name
-    char    subdir[FNAME_SIZE];     // subdir name
-    char    tstring[16];            // actually 12 needed e.g. 200411011230
-    time_t  tstamp;                 // UNIX time stamp
-	int		failed;					// in case of an error
-} srecord_t;
-
-// common_record_t defines ext_map as uint_8, so max 256 extension maps allowed.
-// should be enough anyway
-
-
-typedef struct option_offset_s {
-	struct option_offset_s *next;
-	uint32_t	id;					// table id
-	uint32_t	flags;				// info about this map
-
-	// sampling offsets
-#define HAS_SAMPLER_DATA	1
-	uint16_t	offset_id;
-	uint16_t    sampler_id_length;
-	uint16_t	offset_mode;
-	uint16_t	offset_interval;
-
-#define HAS_STD_SAMPLER_DATA 2
-	uint16_t	offset_std_sampler_interval;
-	uint16_t	offset_std_sampler_algorithm;
-
-} option_offset_t;
-
-typedef struct generic_sampler_s {
-	struct generic_sampler_s *next;
-	sampler_info_record_t	info;
-} generic_sampler_t;
-
-typedef struct generic_exporter_s {
-	// link chain
-	struct generic_exporter_s *next;
-
-	// generic exporter information
-	exporter_info_record_t info;
-
-	uint64_t	packets;			// number of packets sent by this exporter
-	uint64_t	flows;				// number of flow records sent by this exporter
-	uint32_t	sequence_failure;	// number of sequence failues
-
-	generic_sampler_t		*sampler;
-
-} generic_exporter_t;
+/* common minimum netflow header for all versions */
+typedef struct common_flow_header {
+  uint16_t  version;
+  uint16_t  count;
+} common_flow_header_t;
 
 typedef struct FlowSource_s {
 	// link
@@ -114,11 +74,8 @@ typedef struct FlowSource_s {
 	uint64_t			first_seen;		// in msec 
 	uint64_t			last_seen;		// in msec
 
-	// port histogram data
-	xstat_t				*xstat;
-
 	// Any exporter specific data
-	generic_exporter_t	*exporter_data;
+	exporter_t			*exporter_data;
 	uint32_t			exporter_count;
 	struct timeval		received;
 
@@ -131,8 +88,6 @@ typedef struct FlowSource_s {
 		extension_map_t	**maps;
 	} extension_map_list;
 
-	option_offset_t *option_offset_table;
-
 } FlowSource_t;
 
 /* input buffer size, to read data from the network */
@@ -163,10 +118,6 @@ int FlushInfoExporter(FlowSource_t *fs, exporter_info_record_t *exporter);
 
 int FlushInfoSampler(FlowSource_t *fs, sampler_info_record_t *sampler);
 
-int HasOptionTable(FlowSource_t *fs, uint16_t id );
-
-void launcher (char *commbuff, FlowSource_t *FlowSource, char *process, int expire);
-
 /* Default time window in seconds to rotate files */
 #define TIME_WINDOW	  	300
 
