@@ -1,6 +1,5 @@
 /*
- *  Copyright (c) 2014, Peter Haag
- *  Copyright (c) 2009, Peter Haag
+ *  Copyright (c) 2009-2020, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *  
@@ -28,22 +27,17 @@
  *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
  *  POSSIBILITY OF SUCH DAMAGE.
  *  
- *  $Author: haag $
- *
- *  $Id: netflow_v5_v7.c 69 2010-09-09 07:17:43Z haag $
- *
- *  $LastChangedRevision: 69 $
- *	
  */
 
+#ifdef HAVE_CONFIG_H 
 #include "config.h"
+#endif
 
 #include <stdio.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <unistd.h>
 #include <stdlib.h>
-#include <syslog.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <errno.h>
@@ -54,28 +48,23 @@
 #endif
 
 #include "util.h"
+#include "nfdump.h"
 #include "nffile.h"
 #include "nfx.h"
 #include "nfnet.h"
-#include "nf_common.h"
+#include "output_raw.h"
 #include "bookkeeper.h"
-#include "nfxstat.h"
 #include "collector.h"
 #include "exporter.h"
 #include "netflow_v5_v7.h"
 
-#ifndef DEVEL
-#   define dbg_printf(...) /* printf(__VA_ARGS__) */
-#else
-#   define dbg_printf(...) printf(__VA_ARGS__)
-#endif
-
-extern int verbose;
 extern extension_descriptor_t extension_descriptor[];
-extern uint32_t default_sampling;
-extern uint32_t overwrite_sampling;
 
 /* module limited globals */
+static int verbose;
+static uint32_t default_sampling;
+static uint32_t overwrite_sampling;
+
 static extension_info_t v5_extension_info;		// common for all v5 records
 static uint16_t v5_output_record_size, v5_output_record_base_size;
 
@@ -87,19 +76,20 @@ static uint16_t v5_full_mapp[] = { EX_IO_SNMP_2, EX_AS_2, EX_MULIPLE, EX_NEXT_HO
 #define V5_BLOCK_DATA_SIZE (sizeof(ipv4_block_t) - sizeof(uint32_t) + 2 * sizeof(uint64_t))
 
 typedef struct exporter_v5_s {
-	// identical to generic_exporter_t
+	// identical to exporter_t
 	struct exporter_v5_s *next;
 
-	// generic exporter information
+	// exporter information
 	exporter_info_record_t info;
 
 	uint64_t	packets;			// number of packets sent by this exporter
 	uint64_t	flows;				// number of flow records sent by this exporter
 	uint32_t	sequence_failure;	// number of sequence failues
+	uint32_t	padding_errors;		// number of sequence failues
 
-	// generic sampler
-	generic_sampler_t		*sampler;
-	// end of generic_exporter_t
+	// sampler
+	sampler_t		*sampler;
+	// end of exporter_t
 
 	// sequence vars
 	int64_t	 last_sequence;
@@ -126,12 +116,16 @@ static inline int CheckBufferSpace(nffile_t *nffile, size_t required);
 
 #include "nffile_inline.c"
 
-int Init_v5_v7_input(void) {
+int Init_v5_v7_input(int v, uint32_t sampling, uint32_t overwrite) {
 int i, id, map_index;
 int extension_size;
 uint16_t	map_size;
 
-	extension_size   = 0;
+	verbose 		   = v;
+	default_sampling   = sampling;
+	overwrite_sampling = overwrite;
+	extension_size	   = 0;
+
 	// prepare v5 extension map
 	v5_extension_info.map		   = NULL;
 	v5_extension_info.next		   = NULL;
@@ -156,10 +150,10 @@ uint16_t	map_size;
 	if ( ( map_size & 0x3 ) != 0 )
 		map_size += 2;
 
-	// Create a generic v5 extension map
+	// Create a v5 extension map
 	v5_extension_info.map = (extension_map_t *)malloc((size_t)map_size);
 	if ( !v5_extension_info.map ) {
-		syslog(LOG_ERR, "Process_v5: malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror (errno));
+		LogError("Process_v5: malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror (errno));
 		return 0;
 	}
 	v5_extension_info.map->type 	  	  = ExtensionMapType;
@@ -187,7 +181,7 @@ uint16_t	map_size;
 
 static inline exporter_v5_t *GetExporter(FlowSource_t *fs, netflow_v5_header_t *header) {
 exporter_v5_t **e = (exporter_v5_t **)&(fs->exporter_data);
-generic_sampler_t *sampler;
+sampler_t *sampler;
 uint16_t	engine_tag = ntohs(header->engine_tag);
 uint16_t	version    = ntohs(header->version);
 #define IP_STRING_LEN   40
@@ -196,7 +190,7 @@ char ipstr[IP_STRING_LEN];
 	// search the appropriate exporter engine
 	while ( *e ) {
 		if ( (*e)->info.version == version && (*e)->info.id == engine_tag &&
-			 (*e)->info.ip.v6[0] == fs->ip.v6[0] && (*e)->info.ip.v6[1] == fs->ip.v6[1]) 
+			 (*e)->info.ip.V6[0] == fs->ip.V6[0] && (*e)->info.ip.V6[1] == fs->ip.V6[1]) 
 			return *e;
 		e = &((*e)->next);
 	}
@@ -204,7 +198,7 @@ char ipstr[IP_STRING_LEN];
 	// nothing found
 	*e = (exporter_v5_t *)malloc(sizeof(exporter_v5_t));
 	if ( !(*e)) {
-		syslog(LOG_ERR, "Process_v5: malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror (errno));
+		LogError("Process_v5: malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror (errno));
 		return NULL;
 	}
 	memset((void *)(*e), 0, sizeof(exporter_v5_t));
@@ -216,13 +210,14 @@ char ipstr[IP_STRING_LEN];
 	(*e)->info.ip			= fs->ip;
 	(*e)->info.sa_family	= fs->sa_family;
 	(*e)->sequence_failure	= 0;
+	(*e)->padding_errors	= 0;
 	(*e)->packets			= 0;
 	(*e)->flows				= 0;
 	(*e)->first	 			= 1;
 
-	sampler = (generic_sampler_t *)malloc(sizeof(generic_sampler_t));
+	sampler = (sampler_t *)malloc(sizeof(sampler_t));
 	if ( !sampler ) {
-		syslog(LOG_ERR, "Process_v5: malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror (errno));
+		LogError("Process_v5: malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror (errno));
 		return NULL;
 	}
 	(*e)->sampler = sampler;
@@ -238,10 +233,10 @@ char ipstr[IP_STRING_LEN];
 	if ( sampler->info.interval == 0 )
 		sampler->info.interval = default_sampling;
 
-	// copy the v5 generic extension map
+	// copy the v5 extension map
 	(*e)->extension_map		= (extension_map_t *)malloc(v5_extension_info.map->size);
 	if ( !(*e)->extension_map ) {
-		syslog(LOG_ERR, "Process_v5: malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror (errno));
+		LogError("Process_v5: malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror (errno));
 		free(*e);
 		*e = NULL;
 		return NULL;
@@ -262,12 +257,12 @@ char ipstr[IP_STRING_LEN];
 	FlushInfoSampler(fs, &(sampler->info));
 
 	if ( fs->sa_family == AF_INET ) {
-		uint32_t _ip = htonl(fs->ip.v4);
+		uint32_t _ip = htonl(fs->ip.V4);
 		inet_ntop(AF_INET, &_ip, ipstr, sizeof(ipstr));
 	} else if ( fs->sa_family == AF_INET6 ) {
 		uint64_t _ip[2];
-		_ip[0] = htonll(fs->ip.v6[0]);
-		_ip[1] = htonll(fs->ip.v6[1]);
+		_ip[0] = htonll(fs->ip.V6[0]);
+		_ip[1] = htonll(fs->ip.V6[1]);
 		inet_ntop(AF_INET6, &_ip, ipstr, sizeof(ipstr));
 	} else {
 		strncpy(ipstr, "<unknown>", IP_STRING_LEN);
@@ -277,12 +272,12 @@ char ipstr[IP_STRING_LEN];
 
 	dbg_printf("New Exporter: v5 SysID: %u, Extension ID: %i, IP: %s, Sampling Mode: %i, Sampling Interval: %u\n", 
 		(*e)->info.sysid, (*e)->extension_map->map_id, ipstr, sampler->info.mode	,sampler->info.interval);
-	syslog(LOG_INFO, "Process_v5: New exporter: SysID: %u, engine id %u, type %u, IP: %s, Sampling Mode: %i, Sampling Interval: %u\n", 
+	LogInfo("Process_v5: New exporter: SysID: %u, engine id %u, type %u, IP: %s, Sampling Mode: %i, Sampling Interval: %u\n", 
 		(*e)->info.sysid, ( engine_tag & 0xFF ),( (engine_tag >> 8) & 0xFF ), ipstr, sampler->info.mode	,sampler->info.interval );
 
 	if ( overwrite_sampling > 0 )  {
 		sampler->info.interval = overwrite_sampling;
-		syslog(LOG_INFO, "Process_v5: Hard overwrite sampling rate: %u\n", sampler->info.interval);
+		LogInfo("Process_v5: Hard overwrite sampling rate: %u\n", sampler->info.interval);
 	}
 
 	return (*e);
@@ -312,7 +307,7 @@ char		*string;
 
 		exporter = GetExporter(fs, v5_header);
 		if ( !exporter ) {
-			syslog(LOG_ERR,"Process_v5: Exporter NULL: Abort v5/v7 record processing");
+			LogError("Process_v5: Exporter NULL: Abort v5/v7 record processing");
 			return;
 		}
 		exporter->packets++;
@@ -348,14 +343,14 @@ char		*string;
 			// count check
 	  		count	= ntohs(v5_header->count);
 			if ( count > NETFLOW_V5_MAX_RECORDS ) {
-				syslog(LOG_ERR,"Process_v5: Unexpected record count in header: %i. Abort v5/v7 record processing", count);
+				LogError("Process_v5: Unexpected record count in header: %i. Abort v5/v7 record processing", count);
 				fs->nffile->buff_ptr = (void *)common_record;
 				return;
 			}
 
 			// input buffer size check for all expected records
 			if ( size_left < ( NETFLOW_V5_HEADER_LENGTH + count * flow_record_length) ) {
-				syslog(LOG_ERR,"Process_v5: Not enough data to process v5 record. Abort v5/v7 record processing");
+				LogError("Process_v5: Not enough data to process v5 record. Abort v5/v7 record processing");
 				fs->nffile->buff_ptr = (void *)common_record;
 				return;
 			}
@@ -363,7 +358,7 @@ char		*string;
 			// output buffer size check for all expected records
 			if ( !CheckBufferSpace(fs->nffile, count * v5_output_record_size) ) {
 				// fishy! - should never happen. maybe disk full?
-				syslog(LOG_ERR,"Process_v5: output buffer size error. Abort v5/v7 record processing");
+				LogError("Process_v5: output buffer size error. Abort v5/v7 record processing");
 				return;
 			}
 
@@ -389,7 +384,7 @@ char		*string;
 					fs->nffile->stat_record->sequence_failure++;
 					exporter->sequence_failure++;
 					/*
-					syslog(LOG_ERR,"Flow v%d sequence last:%llu now:%llu mismatch. Missing: dist:%lu flows",
+					LogError("Flow v%d sequence last:%llu now:%llu mismatch. Missing: dist:%lu flows",
 						version, exporter->last_sequence, exporter->sequence, exporter->distance);
 					*/
 
@@ -497,7 +492,7 @@ char		*string;
 							} break;
 						case EX_ROUTER_IP_v4:	 {	// IPv4 router address
 							tpl_ext_23_t *tpl = (tpl_ext_23_t *)data_ptr;
-							tpl->router_ip = fs->ip.v4;
+							tpl->router_ip = fs->ip.V4;
 							data_ptr = (void *)tpl->data;
 							ClearFlag(common_record->flags, FLAG_IPV6_EXP);
 							} break;
@@ -516,7 +511,7 @@ char		*string;
 
 						default:
 							// this should never happen, as v5 has no other extensions
-							syslog(LOG_ERR,"Process_v5: Unexpected extension %i for v5 record. Skip extension", id);
+							LogError("Process_v5: Unexpected extension %i for v5 record. Skip extension", id);
 					}
 					j++;
 				}
@@ -534,7 +529,7 @@ char		*string;
 				 */
 				if ( First > Last && ( (First - Last)  < 20000) ) {
 					uint32_t _t;
-					syslog(LOG_ERR,"Process_v5: Unexpected time swap: First 0x%llx smaller than boot time: 0x%llx", start_time, boot_time);
+					LogError("Process_v5: Unexpected time swap: First 0x%llx smaller than boot time: 0x%llx", start_time, boot_time);
 					_t= First;
 					First = Last;
 					Last = _t;
@@ -606,34 +601,11 @@ char		*string;
 				fs->nffile->stat_record->numpackets	+= packets;
 				fs->nffile->stat_record->numbytes	+= bytes;
 
-				if ( fs->xstat ) {
-					uint32_t bpp = packets ? (bytes/packets) : 0;
-					if ( bpp > MAX_BPP ) 
-						bpp = MAX_BPP;
-					if ( common_record->prot == IPPROTO_TCP ) {
-						fs->xstat->bpp_histogram->tcp.bpp[bpp]++;
-						fs->xstat->bpp_histogram->tcp.count++;
-
-						fs->xstat->port_histogram->src_tcp.port[common_record->srcport]++;
-						fs->xstat->port_histogram->dst_tcp.port[common_record->dstport]++;
-						fs->xstat->port_histogram->src_tcp.count++;
-						fs->xstat->port_histogram->dst_tcp.count++;
-					} else if ( common_record->prot == IPPROTO_UDP ) {
-						fs->xstat->bpp_histogram->udp.bpp[bpp]++;
-						fs->xstat->bpp_histogram->udp.count++;
-
-						fs->xstat->port_histogram->src_udp.port[common_record->srcport]++;
-						fs->xstat->port_histogram->dst_udp.port[common_record->dstport]++;
-						fs->xstat->port_histogram->src_udp.count++;
-						fs->xstat->port_histogram->dst_udp.count++;
-					}
-				}
-
-
 				if ( verbose ) {
 					master_record_t master_record;
+					memset((void *)&master_record, 0, sizeof(master_record_t));
 					ExpandRecord_v2((common_record_t *)common_record, &v5_extension_info, &(exporter->info), &master_record);
-				 	format_file_block_record(&master_record, &string, 0);
+				 	flow_record_to_raw(&master_record, &string, 0);
 					printf("%s\n", string);
 				}
 
@@ -652,9 +624,9 @@ char		*string;
 				// buffer size sanity check - should never happen, but check it anyway
 				bsize = (pointer_addr_t)common_record - (pointer_addr_t)fs->nffile->block_header - sizeof(data_block_header_t);
 				if ( bsize >= BUFFSIZE ) {
-					syslog(LOG_ERR,"### Software error ###: %s line %d", __FILE__, __LINE__);
-					syslog(LOG_ERR,"Process_v5: Output buffer overflow! Flush buffer and skip records.");
-					syslog(LOG_ERR,"Buffer size: size: %u, bsize: %llu > %u", fs->nffile->block_header->size, (unsigned long long)bsize, BUFFSIZE);
+					LogError("### Software error ###: %s line %d", __FILE__, __LINE__);
+					LogError("Process_v5: Output buffer overflow! Flush buffer and skip records.");
+					LogError("Buffer size: size: %u, bsize: %llu > %u", fs->nffile->block_header->size, (unsigned long long)bsize, BUFFSIZE);
 					// reset buffer
 					fs->nffile->block_header->size 		= 0;
 					fs->nffile->block_header->NumRecords = 0;
@@ -735,8 +707,8 @@ uint32_t	i, id, t1, t2;
   	v5_output_record->First		= htonl(t1);
   	v5_output_record->Last		= htonl(t2);
 
-	v5_output_record->srcaddr	= htonl(master_record->v4.srcaddr);
-  	v5_output_record->dstaddr	= htonl(master_record->v4.dstaddr);
+	v5_output_record->srcaddr	= htonl(master_record->V4.srcaddr);
+  	v5_output_record->dstaddr	= htonl(master_record->V4.dstaddr);
 
   	v5_output_record->srcport	= htons(master_record->srcport);
   	v5_output_record->dstport	= htons(master_record->dstport);
@@ -748,10 +720,10 @@ uint32_t	i, id, t1, t2;
   	v5_output_record->dPkts		= htonl((uint32_t)master_record->dPkts);
   	v5_output_record->dOctets	= htonl((uint32_t)master_record->dOctets);
 
-  	v5_output_record->input		= 0;
-  	v5_output_record->output	= 0;
-  	v5_output_record->src_as	= 0;
-  	v5_output_record->dst_as	= 0;
+  	v5_output_record->input		= htons(master_record->input);
+  	v5_output_record->output	= htons(master_record->output);
+  	v5_output_record->src_as	= htons(master_record->srcas);
+  	v5_output_record->dst_as	= htons(master_record->dstas);
 	v5_output_record->src_mask 	= 0;
 	v5_output_record->dst_mask 	= 0;
 	v5_output_record->pad1 		= 0;
@@ -774,7 +746,7 @@ uint32_t	i, id, t1, t2;
 				v5_output_record->dst_mask 	= master_record->dst_mask;
 				break;
 			case EX_NEXT_HOP_v4:
-				v5_output_record->nexthop	= htonl(master_record->ip_nexthop.v4);
+				v5_output_record->nexthop	= htonl(master_record->ip_nexthop.V4);
 				break;
 			// default: Other extensions can not be sent with v5
 		}
