@@ -1,7 +1,5 @@
 /*
- *  Copyright (c) 2016, Peter Haag
- *  Copyright (c) 2014, Peter Haag
- *  Copyright (c) 2009, Peter Haag
+ *  Copyright (c) 2009-2020, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *  
@@ -29,19 +27,6 @@
  *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
  *  POSSIBILITY OF SUCH DAMAGE.
  *  
- *  Author: peter
- *
- */
-
-/*
- * Because NetFlow export uses UDP to send export datagrams, it is possible 
- * for datagrams to be lost. To determine whether flow export information has 
- * been lost, Version 5, Version 7, and Version 8 headers contain a flow 
- * sequence number. The sequence number is equal to the sequence number of the 
- * previous datagram plus the number of flows in the previous datagram. After 
- * receiving a new datagram, the receiving application can subtract the expected 
- * sequence number from the sequence number in the header to derive the number 
- * of missed flows.
  */
 
 #include "config.h"
@@ -79,16 +64,16 @@
 #endif
 
 #include "util.h"
+#include "nfdump.h"
 #include "nffile.h"
 #include "nfx.h"
-#include "nf_common.h"
+#include "exporter.h"
 #include "nfnet.h"
 #include "flist.h"
 #include "nfstatfile.h"
 #include "bookkeeper.h"
-#include "nfxstat.h"
+#include "launch.h"
 #include "collector.h"
-#include "exporter.h"
 #include "netflow_v1.h"
 #include "netflow_v5_v7.h"
 #include "netflow_v9.h"
@@ -111,15 +96,10 @@
 #define DEFAULTHOSTNAME "127.0.0.1"
 #define SENDSOCK_BUFFSIZE 200000
 
-#ifndef DEVEL
-#   define dbg_printf(...) /* printf(__VA_ARGS__) */
-#else
-#   define dbg_printf(...) printf(__VA_ARGS__)
-#endif
-
-/* globals */
-caddr_t		shmem;
-int verbose = 0;
+static void *shmem = NULL;
+static int verbose = 0;
+static uint32_t default_sampling   = 1;
+static uint32_t overwrite_sampling = 0;
 
 extern uint32_t default_sampling;   // the default sampling rate when nothing else applies. set by -S
 extern uint32_t overwrite_sampling;	// unconditionally overwrite sampling rate with given sampling rate -S
@@ -148,8 +128,8 @@ static void daemonize(void);
 
 static void SetPriv(char *userid, char *groupid );
 
-static void run(packet_function_t receive_packet, int socket, send_peer_t peer, 
-	time_t twin, time_t t_begin, int report_seq, int use_subdirs, char *time_extension, int compress, int do_xstat);
+static void run(packet_function_t receive_packet, int socket, repeater_t *repeater, 
+	time_t twin, time_t t_begin, int report_seq, int use_subdirs, char *time_extension, int compress);
 
 /* Functions */
 static void usage(char *name) {
@@ -167,21 +147,24 @@ static void usage(char *name) {
 					"-I Ident\tset the ident string for stat file. (default 'none')\n"
 					"-H Add port histogram data to flow file.(default 'no')\n"
 					"-n Ident,IP,logdir\tAdd this flow source - multiple streams\n" 
+					"-M dir \t\tSet the output directory for dynamic sources.\n"
+
 					"-P pidfile\tset the PID file\n"
-					"-R IP[/port]\tRepeat incoming packets to IP address/port\n"
+					"-R IP[/port]\tRepeat incoming packets to IP address/port. Max 8 repeaters.\n"
 					"-s rate\tset default sampling rate (default 1)\n"
 					"-x process\tlaunch process after a new file becomes available\n"
-					"-j\t\tBZ2 compress flows in output file.\n"
 					"-z\t\tLZO compress flows in output file.\n"
+					"-y\t\tLZ4 compress flows in output file.\n"
+					"-j\t\tBZ2 compress flows in output file.\n"
 					"-B bufflen\tSet socket buffer to bufflen bytes\n"
 					"-e\t\tExpire data at each cycle.\n"
 					"-D\t\tFork to background\n"
-					"-E\t\tPrint extended format of netflow data. for debugging purpose only.\n"
+					"-E\t\tPrint extended format of netflow data. For debugging purpose only.\n"
 					"-T\t\tInclude extension tags in records.\n"
 					"-4\t\tListen on IPv4 (default).\n"
 					"-6\t\tListen on IPv6.\n"
 					"-V\t\tPrint version and exit.\n"
-					"-Z\t\tAdd timezone offset to filenamet.\n"
+					"-Z\t\tAdd timezone offset to filename.\n"
 					, name);
 } // End of usage
 
@@ -203,7 +186,7 @@ pid_t ret;
 			sleep(1);
 		}
 		if ( i >= LAUNCHER_TIMEOUT ) {
-			LogError("Laucher does not want to terminate - signal again");
+			LogError("Launcher does not want to terminate - signal again");
 			kill(pid, SIGTERM);
 			sleep(1);
 		}
@@ -356,11 +339,24 @@ int		err;
 
 } // End of SetPriv
 
+static void format_file_block_header(data_block_header_t *header) {
+	
+	printf("\n"
+"File Block Header: \n"
+"  NumBlocks     =  %10u\n"
+"  Size          =  %10u\n"
+"  id         	 =  %10u\n",
+		header->NumRecords,
+		header->size,
+		header->id);
+
+} // End of format_file_block_header
+
 #include "nffile_inline.c"
 #include "collector_inline.c"
 
-static void run(packet_function_t receive_packet, int socket, send_peer_t peer, 
-	time_t twin, time_t t_begin, int report_seq, int use_subdirs, char *time_extension, int compress, int do_xstat) {
+static void run(packet_function_t receive_packet, int socket, repeater_t *repeater, 
+	time_t twin, time_t t_begin, int report_seq, int use_subdirs, char *time_extension, int compress) {
 common_flow_header_t	*nf_header;
 FlowSource_t			*fs;
 struct sockaddr_storage nf_sender;
@@ -372,10 +368,10 @@ uint16_t	version;
 ssize_t		cnt;
 void 		*in_buff;
 int 		err;
-char 		*string;
 srecord_t	*commbuff;
 
-	if ( !Init_v1() || !Init_v5_v7_input() || !Init_v9() || !Init_IPFIX() )
+	if ( !Init_v1(verbose) || !Init_v5_v7_input(verbose, default_sampling, overwrite_sampling) || 
+		 !Init_v9(verbose, default_sampling, overwrite_sampling) || !Init_IPFIX(verbose, default_sampling, overwrite_sampling) )
 		return;
 
 	in_buff  = malloc(NETWORK_INPUT_BUFF_SIZE);
@@ -397,11 +393,6 @@ srecord_t	*commbuff;
 		if ( !fs->nffile ) {
 			return;
 		}
-		if ( do_xstat ) {
-			fs->xstat = InitXStat(fs->nffile);
-			if ( !fs->xstat ) 
-				return;
-		}
 		// init vars
 		fs->bad_packets		= 0;
 		fs->first_seen      = 0xffffffffffffLL;
@@ -418,8 +409,8 @@ srecord_t	*commbuff;
 	periodic_trigger = 0;
 	ignored_packets  = 0;
 
-	// wake up at least at next time slot (twin) + some Overdue time
-	alarm(t_start + twin + OVERDUE_TIME - time(NULL));
+	// wake up at least at next time slot (twin) + 1s
+	alarm(t_start + twin + 1 - time(NULL));
 	/*
 	 * Main processing loop:
 	 * this loop, continues until done = 1, set by the signal handler
@@ -428,6 +419,7 @@ srecord_t	*commbuff;
 	 */
 	while ( 1 ) {
 		struct timeval tv;
+		int i;
 
 		/* read next bunch of data into beginn of input buffer */
 		if ( !done) {
@@ -449,12 +441,15 @@ srecord_t	*commbuff;
 				continue;
 			}
 
-			if ( peer.hostname ) {
+			i = 0;
+			while ( repeater[i].hostname && (i < MAX_REPEATERS)) {
 				ssize_t len;
-				len = sendto(peer.sockfd, in_buff, cnt, 0, (struct sockaddr *)&(peer.addr), peer.addrlen);
+				len = sendto(repeater[i].sockfd, in_buff, cnt, 0, 
+						(struct sockaddr *)&(repeater[i].addr), repeater[i].addrlen);
 				if ( len < 0 ) {
 					LogError("ERROR: sendto(): %s", strerror(errno));
 				}
+				i++;
 			}
 		}
 
@@ -464,13 +459,12 @@ srecord_t	*commbuff;
 		t_now = tv.tv_sec;
 
 		if ( ((t_now - t_start) >= twin) || done ) {
-			char subfilename[64];
 			struct  tm *now;
-			char	*subdir, fmt[64];
+			char	*subdir, fmt[MAXTIMESTRING];
 
 			alarm(0);
 			now = localtime(&t_start);
-			strftime(fmt, sizeof fmt, time_extension, now);
+			strftime(fmt, sizeof(fmt), time_extension, now);
 
 			// prepare sub dir hierarchy
 			if ( use_subdirs ) {
@@ -481,15 +475,10 @@ srecord_t	*commbuff;
 			
 					// failed to generate subdir path - put flows into base directory
 					subdir = NULL;
-					snprintf(subfilename, 63, "nfcapd.%s", fmt);
-				} else {
-					snprintf(subfilename, 63, "%s/nfcapd.%s", subdir, fmt);
-				}
+				} 
 			} else {
 				subdir = NULL;
-				snprintf(subfilename, 63, "nfcapd.%s", fmt);
 			}
-			subfilename[63] = '\0';
 
 			// for each flow source update the stats, close the file and re-initialize the new file
 			fs = FlowSource;
@@ -500,19 +489,29 @@ srecord_t	*commbuff;
 
 				if ( verbose ) {
 					// Dump to stdout
-					format_file_block_header(nffile->block_header, &string, 0);
-					printf("%s\n", string);
+					format_file_block_header(nffile->block_header);
 				}
 
 				if ( nffile->block_header->NumRecords ) {
 					// flush current buffer to disc
 					if ( WriteBlock(nffile) <= 0 )
-						LogError("Ident: %s, failed to write output buffer to disk: '%s'" , fs->Ident, strerror(errno));
+						LogError("Ident: %s, failed to write output buffer to disk: '%s'" , 
+							fs->Ident, strerror(errno));
 				} // else - no new records in current block
 
 	
 				// prepare filename
-				snprintf(nfcapd_filename, MAXPATHLEN-1, "%s/%s", fs->datadir, subfilename);
+				if ( subdir ) {
+					if ( SetupSubDir(fs->datadir, subdir, error, 255) ) {
+						snprintf(nfcapd_filename, MAXPATHLEN-1, "%s/%s/nfcapd.%s", fs->datadir, subdir, fmt);
+					} else {
+						LogError("Ident: %s, Failed to create sub hier directories: %s", fs->Ident, error );
+						// skip subdir - put flows directly into current directory
+						snprintf(nfcapd_filename, MAXPATHLEN-1, "%s/nfcapd.%s", fs->datadir, fmt);
+					}
+				} else {
+					snprintf(nfcapd_filename, MAXPATHLEN-1, "%s/nfcapd.%s", fs->datadir, fmt);
+				}
 				nfcapd_filename[MAXPATHLEN-1] = '\0';
 	
 				// update stat record
@@ -527,25 +526,11 @@ srecord_t	*commbuff;
 				nffile->stat_record->last_seen 	= fs->last_seen/1000;
 				nffile->stat_record->msec_last	= fs->last_seen - nffile->stat_record->last_seen*1000;
 
-				if ( fs->xstat ) {
-					if ( WriteExtraBlock(nffile, fs->xstat->block_header ) <= 0 ) 
-						LogError("Ident: %s, failed to write xstat buffer to disk: '%s'" , fs->Ident, strerror(errno));
-
-					ResetPortHistogram(fs->xstat->port_histogram);
-					ResetBppHistogram(fs->xstat->bpp_histogram);
-				}
-
 				// Flush Exporter Stat to file
 				FlushExporterStats(fs);
 				// Close file
 				CloseUpdateFile(nffile, fs->Ident);
 
-				if ( subdir && !SetupSubDir(fs->datadir, subdir, error, 255) ) {
-					// in this case the flows get lost! - the rename will fail
-					// but this should not happen anyway, unless i/o problems, inode problems etc.
-					LogError("Ident: %s, Failed to create sub hier directories: %s", fs->Ident, error );
-				}
-
 				// if rename fails, we are in big trouble, as we need to get rid of the old .current file
 				// otherwise, we will loose flows and can not continue collecting new flows
 				err = rename(fs->current, nfcapd_filename);
@@ -569,7 +554,8 @@ srecord_t	*commbuff;
 
 				// log stats
 				LogInfo("Ident: '%s' Flows: %llu, Packets: %llu, Bytes: %llu, Sequence Errors: %u, Bad Packets: %u", 
-					fs->Ident, (unsigned long long)nffile->stat_record->numflows, (unsigned long long)nffile->stat_record->numpackets, 
+					fs->Ident, (unsigned long long)nffile->stat_record->numflows, 
+					(unsigned long long)nffile->stat_record->numpackets, 
 					(unsigned long long)nffile->stat_record->numbytes, nffile->stat_record->sequence_failure, fs->bad_packets);
 
 				// reset stats
@@ -583,10 +569,6 @@ srecord_t	*commbuff;
 						LogError("killed due to fatal error: ident: %s", fs->Ident);
 						break;
 					}
-					/* XXX needs fixing */
-					if ( fs->xstat ) {
-						// to be implemented
-					}
 				}
 
 				// Dump all extension maps and exporters to the buffer
@@ -594,23 +576,23 @@ srecord_t	*commbuff;
 
 				// next flow source
 				fs = fs->next;
+
 			} // end of while (fs)
 
-			// All flow sources updated - signal launcher if required
+			// trigger launcher if required
 			if ( launcher_pid ) {
 				// Signal launcher
-		
-				// prepare filename for %f expansion
-				strncpy(commbuff->fname, subfilename, FNAME_SIZE-1);
-				commbuff->fname[FNAME_SIZE-1] = 0;
-				snprintf(commbuff->tstring, 16, "%i%02i%02i%02i%02i", 
-					now->tm_year + 1900, now->tm_mon + 1, now->tm_mday, now->tm_hour, now->tm_min);
-				commbuff->tstring[15] = 0;
+
+				strncpy(commbuff->tstring, fmt, MAXTIMESTRING);
+				commbuff->tstring[MAXTIMESTRING-1] = '\0';
+
 				commbuff->tstamp = t_start;
-				if ( subdir ) 
-					strncpy(commbuff->subdir, subdir, FNAME_SIZE);
-				else
-					commbuff->subdir[0] = '\0';
+				if ( subdir ) {
+					snprintf(commbuff->fname, MAXPATHLEN-1, "%s/nfcapd.%s", subdir, fmt);
+				} else {
+					snprintf(commbuff->fname, MAXPATHLEN-1, "nfcapd.%s", fmt);
+				}
+				commbuff->fname[MAXPATHLEN-1] = '\0';
 
 				if ( launcher_alive ) {
 					LogInfo("Signal launcher");
@@ -630,10 +612,10 @@ srecord_t	*commbuff;
 			t_start += twin;
 			/* t_start = filename time stamp: begin of slot
 		 	* + twin = end of next time interval
-		 	* + OVERDUE_TIME = if no data is collected, this is at latest to act
+		 	* + 1 = act at least 1s after time window expired
 		 	* - t_now = difference value to now
 		 	*/
-			alarm(t_start + twin + OVERDUE_TIME - t_now);
+			alarm(t_start + twin + 1 - t_now);
 
 		}
 
@@ -708,7 +690,7 @@ srecord_t	*commbuff;
 				if ( verbose ) {
 					uint16_t count = ntohs(nf_header->count);
 					if ( blast_cnt != count ) {
-							// LogError("Missmatch blast check: Expected %u got %u\n", blast_cnt, count);
+							// LogError("Mismatch blast check: Expected %u got %u\n", blast_cnt, count);
 						blast_cnt = count;
 						blast_failures++;
 					} else {
@@ -758,27 +740,25 @@ srecord_t	*commbuff;
 
 int main(int argc, char **argv) {
  
-char	*bindhost, *filter, *datadir, pidstr[32], *launch_process;
+char	*bindhost, *datadir, pidstr[32], *launch_process;
 char	*userid, *groupid, *checkptr, *listenport, *mcastgroup, *extension_tags;
 char	*Ident, *dynsrcdir, *time_extension, pidfile[MAXPATHLEN];
 struct stat fstat;
 packet_function_t receive_packet;
-send_peer_t  peer;
+repeater_t repeater[MAX_REPEATERS];
 FlowSource_t *fs;
 struct sigaction act;
 int		family, bufflen;
 time_t 	twin, t_start;
-int		sock, synctime, do_daemonize, expire, spec_time_extension, report_sequence, do_xstat;
+int		sock, do_daemonize, expire, spec_time_extension, report_sequence;
 int		subdir_index, sampling_rate, compress;
-int		c;
+int		c, i;
 #ifdef PCAP
-char	*pcap_file;
- 
-	pcap_file		= NULL;
+char	*pcap_file = NULL;
 #endif
 
 	receive_packet 	= recvfrom;
-	verbose = synctime = do_daemonize = 0;
+	verbose = do_daemonize = 0;
 	bufflen  		= 0;
 	family			= AF_UNSPEC;
 	launcher_pid	= 0;
@@ -788,7 +768,6 @@ char	*pcap_file;
 	bindhost 		= NULL;
 	mcastgroup		= NULL;
 	pidfile[0]		= 0;
-	filter   		= NULL;
 	launch_process	= NULL;
 	userid 			= groupid = NULL;
 	twin	 		= TIME_WINDOW;
@@ -799,15 +778,16 @@ char	*pcap_file;
 	expire			= 0;
 	sampling_rate	= 1;
 	compress		= NOT_COMPRESSED;
-	do_xstat		= 0;
-	memset((void *)&peer, 0, sizeof(send_peer_t));
-	peer.family		= AF_UNSPEC;
+	memset((void *)&repeater, 0, sizeof(repeater));
+	for ( i = 0; i < MAX_REPEATERS; i++ ) {
+		repeater[i].family = AF_UNSPEC;
+	}
 	Ident			= "none";
 	FlowSource		= NULL;
 	extension_tags	= DefaultExtensions;
 	dynsrcdir		= NULL;
 
-	while ((c = getopt(argc, argv, "46ef:whEVI:DB:b:jl:J:M:n:p:P:R:S:s:T:t:x:Xru:g:zZ")) != EOF) {
+	while ((c = getopt(argc, argv, "46ef:whEVI:DB:b:jl:J:M:n:p:P:R:S:s:T:t:x:Xru:g:yzZ")) != EOF) {
 		switch (c) {
 			case 'h':
 				usage(argv[0]);
@@ -843,9 +823,6 @@ char	*pcap_file;
 				printf("%s: Version: %s\n",argv[0], nfdump_version);
 				exit(0);
 				break;
-			case 'X':
-				do_xstat = 1;
-				break;
 			case 'D':
 				do_daemonize = 1;
 				break;
@@ -876,7 +853,7 @@ char	*pcap_file;
 					exit(255);
 				break;
 			case 'w':
-				synctime = 1;
+				// allow for compatibility - always sync timeslot
 				break;
 			case 'B':
 				bufflen = strtol(optarg, &checkptr, 10);
@@ -903,20 +880,34 @@ char	*pcap_file;
 						exit(255);
 					}
 					tmp[MAXPATHLEN-1] = 0;
-					snprintf(pidfile, MAXPATHLEN - 1 - strlen(tmp), "%s/%s", tmp, optarg);
+					if ( (strlen(tmp) + strlen(optarg) + 3) < MAXPATHLEN ) {
+						snprintf(pidfile, MAXPATHLEN - 3 - strlen(tmp), "%s/%s", tmp, optarg);
+					} else {
+						fprintf(stderr, "pidfile MAXPATHLEN error:\n");
+						exit(255);
+					}
 				}
 				// pidfile now absolute path
 				pidfile[MAXPATHLEN-1] = 0;
 				break;
 			case 'R': {
+				char *port, *hostname;
 				char *p = strchr(optarg, '/');
+				int i = 0;
 				if ( p ) { 
 					*p++ = '\0';
-					peer.port = strdup(p);
+					port = strdup(p);
 				} else {
-					peer.port = DEFAULTCISCOPORT;
+					port = DEFAULTCISCOPORT;
 				}
-				peer.hostname = strdup(optarg);
+				hostname = strdup(optarg);
+				while ( repeater[i].hostname && (i < MAX_REPEATERS) ) i++;
+				if ( i == MAX_REPEATERS ) {
+					fprintf(stderr, "Too many packet repeaters! Max: %i repeaters allowed.\n", MAX_REPEATERS);
+					exit(255);
+				}
+				repeater[i].hostname = hostname;
+				repeater[i].port 	 = port;
 
 				break; }
 			case 'r':
@@ -946,6 +937,7 @@ char	*pcap_file;
 					fprintf(stderr, "ERROR: Path too long!\n");
 					exit(255);
 				}
+				datadir = realpath(datadir, NULL);
 				if ( stat(datadir, &fstat) < 0 ) {
 					fprintf(stderr, "stat() failed on %s: %s\n", datadir, strerror(errno));
 					exit(255);
@@ -960,12 +952,12 @@ char	*pcap_file;
 				break;
 			case 't':
 				twin = atoi(optarg);
-				if ( twin <= 0 ) {
-					fprintf(stderr, "ERROR: time frame <= 0\n");
+				if ( twin < 2 ) {
+					LogError("time interval <= 2s not allowed");
 					exit(255);
 				}
 				if (twin < 60) {
-					fprintf(stderr, "WARNING, Very small time frame - < 60s!\n");
+					time_extension	= "%Y%m%d%H%M%S";
 				}
 				break;
 			case 'x':
@@ -973,14 +965,21 @@ char	*pcap_file;
 				break;
 			case 'j':
 				if ( compress ) {
-					LogError("Use either -z for LZO or -j for BZ2 compression, but not both\n");
+					LogError("Use one compression: -z for LZO, -j for BZ2 or -y for LZ4 compression\n");
 					exit(255);
 				}
 				compress = BZ2_COMPRESSED;
 				break;
+			case 'y':
+				if ( compress ) {
+					LogError("Use one compression: -z for LZO, -j for BZ2 or -y for LZ4 compression\n");
+					exit(255);
+				}
+				compress = LZ4_COMPRESSED;
+				break;
 			case 'z':
 				if ( compress ) {
-					LogError("Use either -z for LZO or -j for BZ2 compression, but not both\n");
+					LogError("Use one compression: -z for LZO, -j for BZ2 or -y for LZ4 compression\n");
 					exit(255);
 				}
 				compress = LZO_COMPRESSED;
@@ -1025,7 +1024,7 @@ char	*pcap_file;
 		exit(255);
 	}
 
-	if ( do_daemonize && !InitLog(argv[0], SYSLOG_FACILITY)) {
+	if ( !InitLog(do_daemonize, argv[0], SYSLOG_FACILITY, verbose) ) {
 		exit(255);
 	}
 
@@ -1056,12 +1055,14 @@ char	*pcap_file;
 		exit(255);
 	}
 
-	if ( peer.hostname ) {
-		peer.sockfd = Unicast_send_socket (peer.hostname, peer.port, peer.family, bufflen, 
-											&peer.addr, &peer.addrlen );
-		if ( peer.sockfd <= 0 )
+	i = 0;
+	while ( repeater[i].hostname && (i < MAX_REPEATERS) ) {
+		repeater[i].sockfd = Unicast_send_socket (repeater[i].hostname, repeater[i].port, repeater[i].family, bufflen, 
+											&repeater[i].addr, &repeater[i].addrlen );
+		if ( repeater[i].sockfd <= 0 )
 			exit(255);
-		LogInfo("Replay flows to host: %s port: %s", peer.hostname, peer.port);
+		LogInfo("Replay flows to host: %s port: %s", repeater[i].hostname, repeater[i].port);
+		i++;
 	}
 
 	if ( sampling_rate < 0 ) {
@@ -1121,15 +1122,10 @@ char	*pcap_file;
 		usage(argv[0]);
 		close(sock);
 		exit(255);
-	} else {
-		/* user specified a pcap filter */
-		filter = argv[optind];
-	}
-
+	} 
 
 	t_start = time(NULL);
-	if ( synctime )
-		t_start = t_start - ( t_start % twin);
+	t_start = t_start - ( t_start % twin);
 
 	if ( do_daemonize ) {
 		verbose = 0;
@@ -1152,14 +1148,11 @@ char	*pcap_file;
 
 	done = 0;
 	if ( launch_process || expire ) {
-		// for efficiency reason, the process collecting the data
-		// and the process launching processes, when a new file becomes
-		// available are separated. Communication is done using signals
-		// as well as shared memory
-		// prepare shared memory
+
+		// create laucher comm memory struct
 		shmem = mmap(0, sizeof(srecord_t), PROT_READ | PROT_WRITE, MAP_ANON | MAP_SHARED, -1, 0);
-		if ( shmem == (caddr_t)-1 ) {
-			LogError("mmap() error: %s", strerror(errno));
+		if ( shmem == MAP_FAILED) {
+			LogError("mmap() error in %s:%i: %s", __FILE__, __LINE__ , strerror(errno));
 			close(sock);
 			exit(255);
 		}
@@ -1169,7 +1162,7 @@ char	*pcap_file;
 			case 0:
 				// child
 				close(sock);
-				launcher((char *)shmem, FlowSource, launch_process, expire);
+				launcher(shmem, FlowSource, launch_process, expire);
 				_exit(0);
 				break;
 			case -1:
@@ -1225,8 +1218,8 @@ char	*pcap_file;
 	sigaction(SIGCHLD, &act, NULL);
 
 	LogInfo("Startup.");
-	run(receive_packet, sock, peer, twin, t_start, report_sequence, subdir_index, 
-		time_extension, compress, do_xstat);
+	run(receive_packet, sock, repeater, twin, t_start, report_sequence, subdir_index, 
+		time_extension, compress);
 	close(sock);
 	kill_launcher(launcher_pid);
 
