@@ -1,6 +1,5 @@
 /*
- *  Copyright (c) 2014, Peter Haag
- *  Copyright (c) 2009, Peter Haag
+ *  Copyright (c) 2009-2020, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *  
@@ -28,13 +27,6 @@
  *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
  *  POSSIBILITY OF SUCH DAMAGE.
  *  
- *  $Author: haag $
- *
- *  $Id: nfdump.c 69 2010-09-09 07:17:43Z haag $
- *
- *  $LastChangedRevision: 69 $
- *	
- *
  */
 
 #include "config.h"
@@ -59,32 +51,28 @@
 #include <stdint.h>
 #endif
 
+#include "util.h"
+#include "nfdump.h"
 #include "nffile.h"
 #include "nfx.h"
+#include "flist.h"
 #include "nfnet.h"
 #include "bookkeeper.h"
-#include "nfxstat.h"
 #include "collector.h"
 #include "exporter.h"
-#include "nf_common.h"
+#include "output_util.h"
+#include "output_raw.h"
+#include "output_pipe.h"
+#include "output_csv.h"
+#include "output_json.h"
 #include "netflow_v5_v7.h"
 #include "netflow_v9.h"
-#include "rbtree.h"
 #include "nftree.h"
 #include "nfprof.h"
-#include "nfdump.h"
 #include "nflowcache.h"
 #include "nfstat.h"
 #include "nfexport.h"
 #include "ipconv.h"
-#include "util.h"
-#include "flist.h"
-
-#ifndef DEVEL
-#   define dbg_printf(...) /* printf(__VA_ARGS__) */
-#else
-#   define dbg_printf(...) printf(__VA_ARGS__)
-#endif
 
 /* hash parameters */
 #define NumPrealloc 128000
@@ -92,20 +80,16 @@
 #define AGGR_SIZE 7
 
 /* Global Variables */
-FilterEngine_data_t	*Engine;
+FilterEngine_t	*Engine;
 
 extern char	*FilterFilename;
 extern uint32_t loopcnt;
 
-#ifdef COMPAT15
-extern extension_descriptor_t extension_descriptor[];
-#endif
-
 /* Local Variables */
 const char *nfdump_version = VERSION;
 
 static uint64_t total_bytes;
-static uint32_t total_flows;
+static uint32_t recordCount;
 static uint32_t skipped_blocks;
 static uint32_t	is_anonymized;
 static time_t 	t_first_flow, t_last_flow;
@@ -118,7 +102,7 @@ int hash_skip = 0;
 
 extension_map_list_t *extension_map_list;
 
-extern generic_exporter_t **exporter_list;
+extern exporter_t **exporter_list;
 /*
  * Output Formats:
  * User defined output formats can be compiled into nfdump, for easy access
@@ -222,27 +206,30 @@ extern generic_exporter_t **exporter_list;
  * 5. Recompile nfdump
  */
 
+static void flow_record_to_null(void *record, char ** s, int tag);
+
 // Assign print functions for all output options -o
 // Teminated with a NULL record
 printmap_t printmap[] = {
-	{ "raw",		format_file_block_record,  	NULL 			},
-	{ "line", 		format_special,      		FORMAT_line 	},
-	{ "long", 		format_special, 			FORMAT_long 	},
-	{ "extended",	format_special, 			FORMAT_extended	},
-	{ "biline", 	format_special,      		FORMAT_biline 	},
-	{ "bilong", 	format_special,      		FORMAT_bilong 	},
-	{ "pipe", 		flow_record_to_pipe,      	NULL 			},
-	{ "csv", 		flow_record_to_csv,      	NULL 			},
-	{ "null", 		flow_record_to_null,      	NULL 			},
+	{ "raw",		flow_record_to_raw,  		raw_prolog, raw_epilog, NULL },
+	{ "line", 		format_special,      		text_prolog, text_epilog, FORMAT_line },
+	{ "long", 		format_special, 			text_prolog, text_epilog, FORMAT_long },
+	{ "extended",	format_special, 			text_prolog, text_epilog, FORMAT_extended },
+	{ "biline", 	format_special,      		text_prolog, text_epilog, FORMAT_biline	},
+	{ "bilong", 	format_special,      		text_prolog, text_epilog, FORMAT_bilong	},
+	{ "pipe", 		flow_record_to_pipe,      	pipe_prolog, pipe_epilog, NULL },
+	{ "json", 		flow_record_to_json,      	json_prolog, json_epilog, NULL },
+	{ "csv", 		flow_record_to_csv,      	csv_prolog,  csv_epilog,  NULL },
+	{ "null", 		flow_record_to_null,      	text_prolog, text_epilog, NULL },
 #ifdef NSEL
-	{ "nsel",		format_special, 			FORMAT_nsel		},
-	{ "nel",		format_special, 			FORMAT_nel		},
+	{ "nsel",		format_special, 			text_prolog, text_epilog, FORMAT_nsel },
+	{ "nel",		format_special, 			text_prolog, text_epilog, FORMAT_nel },
 #endif
 
 // add your formats here
 
 // This is always the last line
-	{ NULL,			NULL,                       NULL			}
+	{ NULL, NULL, NULL, NULL, "" }
 };
 
 // For automatic output format generation in case of custom aggregation
@@ -255,11 +242,11 @@ printmap_t printmap[] = {
 /* Function Prototypes */
 static void usage(char *name);
 
-static void PrintSummary(stat_record_t *stat_record, int plain_numbers, int csv_output);
+static void PrintSummary(stat_record_t *stat_record, int printPlain, int csv_output);
 
 static stat_record_t process_data(char *wfile, int element_stat, int flow_stat, int sort_flows,
-	printer_t print_header, printer_t print_record, time_t twin_start, time_t twin_end, 
-	uint64_t limitflows, int tag, int compress, int do_xstat);
+	printer_t print_record, time_t twin_start, time_t twin_end, 
+	uint64_t limitRecords, int tag, int compress);
 
 /* Functions */
 
@@ -285,11 +272,11 @@ static void usage(char *name) {
 					"-s <expr>[/<order>]\tGenerate statistics for <expr> any valid record element.\n"
 					"\t\tand ordered by <order>: packets, bytes, flows, bps pps and bpp.\n"
 					"-q\t\tQuiet: Do not print the header and bottom stat lines.\n"
-					"-H Add xstat histogram data to flow file.(default 'no')\n"
 					"-i <ident>\tChange Ident to <ident> in file given by -r.\n"
-					"-J <num>\tModify file compression: 0: uncompressed - 1: LZO compressed - 2: BZ2 compressed.\n"
-					"-z\t\tlzo compress flows in output file. Used in combination with -w.\n"
-					"-j\t\tbz2 compress flows in output file. Used in combination with -w.\n"
+					"-J <num>\tModify file compression: 0: uncompressed - 1: LZO - 2: BZ2 - 3: LZ4 compressed.\n"
+					"-z\t\tLZO compress flows in output file. Used in combination with -w.\n"
+					"-y\t\tLZ4 compress flows in output file. Used in combination with -w.\n"
+					"-j\t\tBZ2 compress flows in output file. Used in combination with -w.\n"
 					"-l <expr>\tSet limit on packets for line and packed output format.\n"
 					"\t\tkey: 32 character string or 64 digit hex string starting with 0x.\n"
 					"-L <expr>\tSet limit on bytes for line and packed output format.\n"
@@ -297,7 +284,7 @@ static void usage(char *name) {
 					"-M <expr>\tRead input from multiple directories.\n"
 					"\t\t/dir/dir1:dir2:dir3 Read the same files from '/dir/dir1' '/dir/dir2' and '/dir/dir3'.\n"
 					"\t\trequests either -r filename or -R firstfile:lastfile without pathnames\n"
-					"-m\t\tdepricated\n"
+					"-m\t\tdeprecated\n"
 					"-O <order> Sort order for aggregated flows - tstart, tend, flows, packets bps pps bbp etc.\n"
 					"-R <expr>\tRead input from sequence of files.\n"
 					"\t\t/any/dir  Read all files in that directory.\n"
@@ -309,9 +296,11 @@ static void usage(char *name) {
 					"\t\t long     Standard output line format with additional fields.\n"
 					"\t\t extended Even more information.\n"
 					"\t\t csv      ',' separated, machine parseable output format.\n"
+					"\t\t json     json output format.\n"
 					"\t\t pipe     '|' separated legacy machine parseable output format.\n"
+					"\t\t null     no flow records, but statistics output.\n"
 					"\t\t\tmode may be extended by '6' for full IPv6 listing. e.g.long6, extended6.\n"
-					"-E <file>\tPrint exporter ans sampling info for collected flows.\n"
+					"-E <file>\tPrint exporter and sampling info for collected flows.\n"
 					"-v <file>\tverify netflow data file. Print version and blocks.\n"
 					"-x <file>\tverify extension records in netflow data file.\n"
 					"-X\t\tDump Filtertable and exit (debug option).\n"
@@ -320,8 +309,11 @@ static void usage(char *name) {
 					"\t\tyyyy/MM/dd.hh:mm:ss[-yyyy/MM/dd.hh:mm:ss]\n", name);
 } /* usage */
 
+static void flow_record_to_null(void *record, char ** s, int tag) {
+	// empty - do not list any flows
+} // End of flow_record_to_null
 
-static void PrintSummary(stat_record_t *stat_record, int plain_numbers, int csv_output) {
+static void PrintSummary(stat_record_t *stat_record, int printPlain, int csv_output) {
 static double	duration;
 uint64_t	bps, pps, bpp;
 char 		byte_str[NUMBER_STRING_SIZE], packet_str[NUMBER_STRING_SIZE];
@@ -347,17 +339,19 @@ char 		bps_str[NUMBER_STRING_SIZE], pps_str[NUMBER_STRING_SIZE], bpp_str[NUMBER_
 			(long long unsigned)stat_record->numflows, (long long unsigned)stat_record->numbytes, 
 			(long long unsigned)stat_record->numpackets, (long long unsigned)bps, 
 			(long long unsigned)pps, (long long unsigned)bpp );
-	} else if ( plain_numbers ) {
+/*
+	} else if ( printPlain ) {
 		printf("Summary: total flows: %llu, total bytes: %llu, total packets: %llu, avg bps: %llu, avg pps: %llu, avg bpp: %llu\n",
 			(long long unsigned)stat_record->numflows, (long long unsigned)stat_record->numbytes, 
 			(long long unsigned)stat_record->numpackets, (long long unsigned)bps, 
 			(long long unsigned)pps, (long long unsigned)bpp );
+*/
 	} else {
-		format_number(stat_record->numbytes, byte_str, DONT_SCALE_NUMBER, VAR_LENGTH);
-		format_number(stat_record->numpackets, packet_str, DONT_SCALE_NUMBER, VAR_LENGTH);
-		format_number(bps, bps_str, DONT_SCALE_NUMBER, VAR_LENGTH);
-		format_number(pps, pps_str, DONT_SCALE_NUMBER, VAR_LENGTH);
-		format_number(bpp, bpp_str, DONT_SCALE_NUMBER, VAR_LENGTH);
+		format_number(stat_record->numbytes, byte_str, printPlain, VAR_LENGTH);
+		format_number(stat_record->numpackets, packet_str, printPlain, VAR_LENGTH);
+		format_number(bps, bps_str, printPlain, VAR_LENGTH);
+		format_number(pps, pps_str, printPlain, VAR_LENGTH);
+		format_number(bpp, bpp_str, printPlain, VAR_LENGTH);
 		printf("Summary: total flows: %llu, total bytes: %s, total packets: %s, avg bps: %s, avg pps: %s, avg bpp: %s\n",
 		(unsigned long long)stat_record->numflows, byte_str, packet_str, bps_str, pps_str, bpp_str );
 	}
@@ -365,19 +359,14 @@ char 		bps_str[NUMBER_STRING_SIZE], pps_str[NUMBER_STRING_SIZE], bpp_str[NUMBER_
 } // End of PrintSummary
 
 stat_record_t process_data(char *wfile, int element_stat, int flow_stat, int sort_flows,
-	printer_t print_header, printer_t print_record, time_t twin_start, time_t twin_end, 
-	uint64_t limitflows, int tag, int compress, int do_xstat) {
+	printer_t print_record, time_t twin_start, time_t twin_end, 
+	uint64_t limitRecords, int tag, int compress) {
 common_record_t 	*flow_record, *record_ptr;
 master_record_t		*master_record;
 nffile_t			*nffile_w, *nffile_r;
-xstat_t				*xstat;
 stat_record_t 		stat_record;
 int 				done, write_file;
 
-#ifdef COMPAT15
-int	v1_map_done = 0;
-#endif
-	
 	// time window of all matched flows
 	memset((void *)&stat_record, 0, sizeof(stat_record_t));
 	stat_record.first_seen = 0x7fffffff;
@@ -395,7 +384,6 @@ int	v1_map_done = 0;
 	write_file = !(sort_flows || flow_stat || element_stat) && wfile;
 	nffile_r = NULL;
 	nffile_w = NULL;
-	xstat  	 = NULL;
 
 	// Get the first file handle
 	nffile_r = GetNextFile(NULL, twin_start, twin_end);
@@ -429,16 +417,6 @@ int	v1_map_done = 0;
 			}
 			return stat_record;
 		}
-		if ( do_xstat ) {
-			xstat = InitXStat(nffile_w);
-			if ( !xstat ) {
-				if ( nffile_r ) {
-					CloseFile(nffile_r);
-					DisposeFile(nffile_r);
-				}
-				return stat_record;
-			}
-		}
 	}
 
 	// setup Filter Engine to point to master_record, as any record read from file
@@ -448,8 +426,6 @@ int	v1_map_done = 0;
 	done = 0;
 	while ( !done ) {
 	int i, ret;
-	char *ConvertBuffer = NULL;
-
 		// get next data block from file
 		ret = ReadBlock(nffile_r);
 
@@ -484,61 +460,9 @@ int	v1_map_done = 0;
 				total_bytes += ret;
 		}
 
-
-#ifdef COMPAT15
-		if ( nffile_r->block_header->id == DATA_BLOCK_TYPE_1 ) {
-			common_record_v1_t *v1_record = (common_record_v1_t *)nffile_r->buff_ptr;
-			// create an extension map for v1 blocks
-			if ( v1_map_done == 0 ) {
-				extension_map_t *map = malloc(sizeof(extension_map_t) + 2 * sizeof(uint16_t) );
-				if ( ! map ) {
-					LogError("malloc() allocation error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
-					exit(255);
-				}
-				map->type 	= ExtensionMapType;
-				map->size 	= sizeof(extension_map_t) + 2 * sizeof(uint16_t);
-				if (( map->size & 0x3 ) != 0 ) {
-					map->size += 4 - ( map->size & 0x3 );
-				}
-
-				map->map_id = INIT_ID;
-
-				map->ex_id[0]  = EX_IO_SNMP_2;
-				map->ex_id[1]  = EX_AS_2;
-				map->ex_id[2]  = 0;
-				
-				map->extension_size  = 0;
-				map->extension_size += extension_descriptor[EX_IO_SNMP_2].size;
-				map->extension_size += extension_descriptor[EX_AS_2].size;
-
-				if ( Insert_Extension_Map(extension_map_list,map) && write_file ) {
-					// flush new map
-					AppendToBuffer(nffile_w, (void *)map, map->size);
-				} // else map already known and flushed
-
-				v1_map_done = 1;
-			}
-
-			// convert the records to v2
-			for ( i=0; i < nffile_r->block_header->NumRecords; i++ ) {
-				common_record_t *v2_record = (common_record_t *)v1_record;
-				Convert_v1_to_v2((void *)v1_record);
-				// now we have a v2 record -> use size of v2_record->size
-				v1_record = (common_record_v1_t *)((pointer_addr_t)v1_record + v2_record->size);
-			}
-			nffile_r->block_header->id = DATA_BLOCK_TYPE_2;
-		}
-#endif
-
-		if ( nffile_r->block_header->id == Large_BLOCK_Type ) {
-			// skip
-			printf("Xstat block skipped ...\n");
-			continue;
-		}
-
 		if ( nffile_r->block_header->id != DATA_BLOCK_TYPE_2 ) {
 			if ( nffile_r->block_header->id == DATA_BLOCK_TYPE_1 ) {
-				LogError("Can't process nfdump 1.5.x block type 1. Add --enable-compat15 to compile compatibility code. Skip block.\n");
+				LogError("nfdump 1.5.x block type 1 no longer supported. Skip block.\n");
 			} else {
 				LogError("Can't process block type %u. Skip block.\n", nffile_r->block_header->id);
 			}
@@ -546,26 +470,23 @@ int	v1_map_done = 0;
 			continue;
 		}
 
+		uint32_t sumSize = 0;
 		record_ptr = nffile_r->buff_ptr;
 		for ( i=0; i < nffile_r->block_header->NumRecords; i++ ) {
 			flow_record = record_ptr;
+			if ( (sumSize + record_ptr->size) > ret || (record_ptr->size < sizeof(record_header_t)) ) {
+				LogError("Corrupt data file. Inconsistent block size in %s line %d\n", __FILE__, __LINE__);
+				exit(255);
+			}
+			sumSize += record_ptr->size;
 			switch ( record_ptr->type ) {
 				case CommonRecordV0Type: 
-					// convert common record v0
-					if ( !ConvertBuffer ) {
-						ConvertBuffer = malloc(65536);
-						if ( !ConvertBuffer ) {
-							LogError("malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
-							exit(255);
-						}
-					}
-					ConvertCommonV0((void *)record_ptr, (common_record_t *)ConvertBuffer);
-					flow_record = (common_record_t *)ConvertBuffer;
-					dbg_printf("Converted type %u to %u record\n", CommonRecordV0Type, CommonRecordType);
+					LogError("Old common v0 records no longer supported - skipped");
+					break;
 				case CommonRecordType: {
 					int match;
 					uint32_t map_id;
-					generic_exporter_t *exp_info;
+					exporter_t *exp_info;
 
 					// valid flow_record converted if needed
 					map_id = flow_record->ext_map;
@@ -581,7 +502,7 @@ int	v1_map_done = 0;
 						continue;
 					} 
 
-					total_flows++;
+					recordCount++;
 					master_record = &(extension_map_list->slot[map_id]->master_record);
 					Engine->nfrecord = (uint64_t *)master_record;
 					ExpandRecord_v2( flow_record, extension_map_list->slot[map_id], 
@@ -590,7 +511,7 @@ int	v1_map_done = 0;
 					// Time based filter
 					// if no time filter is given, the result is always true
 					match  = twin_start && (master_record->first < twin_start || master_record->last > twin_end) ? 0 : 1;
-					match &= limitflows ? stat_record.numflows < limitflows : 1;
+					match &= limitRecords ? recordCount <= limitRecords : 1;
 
 					// filter netflow record with user supplied filter
 					if ( match ) 
@@ -605,6 +526,11 @@ int	v1_map_done = 0;
 
 					// Records passed filter -> continue record processing
 					// Update statistics
+					master_record->label = Engine->label;
+#ifdef DEVEL
+					if ( Engine->label )
+						printf("Flow has label: %s\n", Engine->label);
+#endif
 					UpdateStat(&stat_record, master_record);
 
 					// update number of flows matching a given map
@@ -622,18 +548,12 @@ int	v1_map_done = 0;
 					} else {
 						if ( write_file ) {
 							AppendToBuffer(nffile_w, (void *)flow_record, flow_record->size);
-							if ( xstat ) 
-								UpdateXStat(xstat, master_record);
 						} else if ( print_record ) {
 							char *string;
 							// if we need to print out this record
 							print_record(master_record, &string, tag);
 							if ( string ) {
-								if ( limitflows ) {
-									if ( (stat_record.numflows <= limitflows) )
-										printf("%s\n", string);
-								} else 
-									printf("%s\n", string);
+								printf("%s\n", string);
 							}
 						} else { 
 							// mutually exclusive conditions should prevent executing this code
@@ -645,14 +565,22 @@ int	v1_map_done = 0;
 				case ExtensionMapType: {
 					extension_map_t *map = (extension_map_t *)record_ptr;
 	
-					if ( Insert_Extension_Map(extension_map_list, map) && write_file ) {
-						// flush new map
-						AppendToBuffer(nffile_w, (void *)map, map->size);
-					} // else map already known and flushed
+					int ret = Insert_Extension_Map(extension_map_list, map);
+					switch (ret) {
+						case 0:
+							break; // map already known and flushed
+						case 1:
+							if ( write_file ) 
+								AppendToBuffer(nffile_w, (void *)map, map->size);
+							break;
+						default:
+							LogError("Corrupt data file. Unable to decode at %s line %d\n", __FILE__, __LINE__);
+							exit(255);
+					}
 					} break;
-				case ExporterRecordType:
-				case SamplerRecordype:
-						// Silently skip exporter records
+				case LegacyRecordType1:
+				case LegacyRecordType2:
+						// Silently skip legacy records
 					break;
 				case ExporterInfoRecordType: {
 					int ret = AddExporterInfo((exporter_info_record_t *)record_ptr);
@@ -687,8 +615,8 @@ int	v1_map_done = 0;
 		} // for all records
 
 		// check if we are done, due to -c option 
-		if ( limitflows ) 
-			done = stat_record.numflows >= limitflows;
+		if ( limitRecords ) 
+			done = recordCount >= limitRecords;
 
 	} // while
 
@@ -703,12 +631,6 @@ int	v1_map_done = 0;
 			} 
 		}
 
-		if ( xstat ) {
-			if ( WriteExtraBlock(nffile_w, xstat->block_header ) <= 0 ) {
-				LogError("Failed to write xstat buffer to disk: '%s'" , strerror(errno));
-			} 
-		}
-
 		/* Stat info */
 		if ( write_file ) {
 			/* Copy stat info and close file */
@@ -729,17 +651,19 @@ int	v1_map_done = 0;
 int main( int argc, char **argv ) {
 struct stat stat_buff;
 stat_record_t	sum_stat;
-printer_t 	print_header, print_record;
+printer_t 	print_record;
+func_prolog_t print_prolog;
+func_epilog_t print_epilog;
 nfprof_t 	profile_data;
 char 		*rfile, *Rfile, *Mdirs, *wfile, *ffile, *filter, *tstring, *stat_type;
-char		*byte_limit_string, *packet_limit_string, *print_format, *record_header;
+char		*byte_limit_string, *packet_limit_string, *print_format;
 char		*print_order, *query_file, *nameserver, *aggr_fmt;
 int 		c, ffd, ret, element_stat, fdump;
-int 		i, user_format, quiet, flow_stat, topN, aggregate, aggregate_mask, bidir;
-int 		print_stat, syntax_only, date_sorted, do_tag, compress, do_xstat;
-int			plain_numbers, GuessDir, pipe_output, csv_output, ModifyCompress;
+int 		i, quiet, flow_stat, topN, aggregate, aggregate_mask, bidir;
+int 		print_stat, syntax_only, date_sorted, do_tag, compress;
+int			printPlain, GuessDir, pipe_output, csv_output, json_output, ModifyCompress;
 time_t 		t_start, t_end;
-uint32_t	limitflows;
+uint32_t	limitRecords;
 char 		Ident[IDENTLEN];
 
 	rfile = Rfile = Mdirs = wfile = ffile = filter = tstring = stat_type = NULL;
@@ -753,35 +677,34 @@ char 		Ident[IDENTLEN];
 	flow_stat       = 0;
 	print_stat      = 0;
 	element_stat  	= 0;
-	do_xstat 		= 0;
-	limitflows		= 0;
+	limitRecords		= 0;
 	date_sorted		= 0;
 	total_bytes		= 0;
-	total_flows		= 0;
+	recordCount		= 0;
 	skipped_blocks	= 0;
 	do_tag			= 0;
 	quiet			= 0;
-	user_format		= 0;
 	compress		= NOT_COMPRESSED;
-	plain_numbers   = 0;
+	printPlain		= 0;
 	pipe_output		= 0;
 	csv_output		= 0;
+	json_output		= 0;
 	is_anonymized	= 0;
 	GuessDir		= 0;
 	nameserver		= NULL;
 
 	print_format    = NULL;
-	print_header 	= NULL;
 	print_record  	= NULL;
+	print_prolog	= NULL;
+	print_epilog	= NULL;
 	print_order  	= NULL;
 	query_file		= NULL;
 	ModifyCompress	= -1;
 	aggr_fmt		= NULL;
-	record_header 	= NULL;
 
 	Ident[0] = '\0';
 
-	while ((c = getopt(argc, argv, "6aA:Bbc:D:E:s:hHn:i:jf:qzr:v:w:J:K:M:NImO:R:XZt:TVv:x:l:L:o:")) != EOF) {
+	while ((c = getopt(argc, argv, "6aA:Bbc:D:E:s:hn:i:jf:qyzr:v:w:J:K:M:NImO:R:XZt:TVv:x:l:L:o:")) != EOF) {
 		switch (c) {
 			case 'h':
 				usage(argv[0]);
@@ -831,21 +754,28 @@ char 		Ident[IDENTLEN];
 				break;
 			case 'j':
 				if ( compress ) {
-					LogError("Use either -z for LZO or -j for BZ2 compression, but not both\n");
+					LogError("Use one compression: -z for LZO, -j for BZ2 or -y for LZ4 compression\n");
 					exit(255);
 				}
 				compress = BZ2_COMPRESSED;
 				break;
+			case 'y':
+				if ( compress ) {
+					LogError("Use one compression: -z for LZO, -j for BZ2 or -y for LZ4 compression\n");
+					exit(255);
+				}
+				compress = LZ4_COMPRESSED;
+				break;
 			case 'z':
 				if ( compress ) {
-					LogError("Use either -z for LZO or -j for BZ2 compression, but not both\n");
+					LogError("Use one compression: -z for LZO, -j for BZ2 or -y for LZ4 compression\n");
 					exit(255);
 				}
 				compress = LZO_COMPRESSED;
 				break;
 			case 'c':	
-				limitflows = atoi(optarg);
-				if ( !limitflows ) {
+				limitRecords = atoi(optarg);
+				if ( !limitRecords ) {
 					LogError("Option -c needs a number > 0\n");
 					exit(255);
 				}
@@ -873,14 +803,11 @@ char 		Ident[IDENTLEN];
 				LogError("*** Anonymisation moved! Use nfanon to anonymise flows!\n");
 				exit(255);
 				break;
-			case 'H':
-				do_xstat = 1;
-				break;
 			case 'L':
 				byte_limit_string = optarg;
 				break;
 			case 'N':
-				plain_numbers = 1;
+				printPlain = 1;
 				break;
 			case 'f':
 				ffile = optarg;
@@ -897,7 +824,7 @@ char 		Ident[IDENTLEN];
 				print_order = "tstart";
 				Parse_PrintOrder(print_order);
 				date_sorted = 1;
-				LogError("Option -m depricated. Use '-O tstart' instead\n");
+				LogError("Option -m deprecated. Use '-O tstart' instead\n");
 				break;
 			case 'M':
 				Mdirs = optarg;
@@ -907,6 +834,11 @@ char 		Ident[IDENTLEN];
 				break;
 			case 'o':	// output mode
 				print_format = optarg;
+				// limit input chars
+				if ( strlen(print_format) > 512 ) {
+					LogError("Length of ouput format string too big - > 512\n");
+					exit(255);
+				}
 				break;
 			case 'O': {	// stat order by
 				int ret;
@@ -935,8 +867,8 @@ char 		Ident[IDENTLEN];
 				do_tag = 1;
 				break;
 			case 'i':
-				strncpy(Ident, optarg, IDENT_SIZE);
-				Ident[IDENT_SIZE - 1] = 0;
+				strncpy(Ident, optarg, IDENTLEN);
+				Ident[IDENTLEN - 1] = 0;
 				if ( strchr(Ident, ' ') ) {
 					LogError("Ident must not contain spaces\n");
 					exit(255);
@@ -944,8 +876,8 @@ char 		Ident[IDENTLEN];
 				break;
 			case 'J':
 				ModifyCompress = atoi(optarg);
-				if ( (ModifyCompress < 0) || (ModifyCompress > 2) ) {
-					LogError("Expected -J <num>, 0: uncompressed, 1: LZO compressed, 2: BZ2 compressed.\n");
+				if ( (ModifyCompress < 0) || (ModifyCompress > 3) ) {
+					LogError("Expected -J <num>, 0: uncompressed, 1: LZO, 2: BZ2, 3: LZ4 compressed.\n");
 					exit(255);
 				}
 				break;
@@ -1009,7 +941,7 @@ char 		Ident[IDENTLEN];
 	}
 
 	if ( rfile && Rfile ) {
-		LogError("-r and -R are mutually exclusive. Plase specify either -r or -R\n");
+		LogError("-r and -R are mutually exclusive. Please specify either -r or -R\n");
 		exit(255);
 	}
 	if ( Mdirs && !(rfile || Rfile) ) {
@@ -1070,11 +1002,10 @@ char 		Ident[IDENTLEN];
 		// special user defined output format
 		char *format = &print_format[4];
 		if ( strlen(format) ) {
-			if ( !ParseOutputFormat(format, plain_numbers, printmap) )
+			if ( !ParseOutputFormat(format, printPlain, printmap) )
 				exit(255);
 			print_record  = format_special;
-			record_header = get_record_header();
-			user_format	  = 1;
+			print_prolog  = text_prolog;
 		} else {
 			LogError("Missing format description for user defined output format!\n");
 			exit(255);
@@ -1096,12 +1027,12 @@ char 		Ident[IDENTLEN];
 		while ( printmap[i].printmode ) {
 			if ( strncasecmp(print_format, printmap[i].printmode, MAXMODELEN) == 0 ) {
 				if ( printmap[i].Format ) {
-					if ( !ParseOutputFormat(printmap[i].Format, plain_numbers, printmap) )
+					if ( !ParseOutputFormat(printmap[i].Format, printPlain, printmap) )
 						exit(255);
 					// predefined custom format
-					print_record  = printmap[i].func;
-					record_header = get_record_header();
-					user_format	  = 1;
+					print_record  = printmap[i].func_record;
+					print_prolog  = printmap[i].func_prolog;
+					print_epilog  = printmap[i].func_epilog;
 				} else {
 					// To support the pipe output format for element stats - check for pipe, and remember this
 					if ( strncasecmp(print_format, "pipe", MAXMODELEN) == 0 ) {
@@ -1109,12 +1040,14 @@ char 		Ident[IDENTLEN];
 					}
 					if ( strncasecmp(print_format, "csv", MAXMODELEN) == 0 ) {
 						csv_output = 1;
-						set_record_header();
-						record_header = get_record_header();
+					}
+					if ( strncasecmp(print_format, "json", MAXMODELEN) == 0 ) {
+						json_output = 1;
 					}
 					// predefined static format
-					print_record  = printmap[i].func;
-					user_format	  = 0;
+					print_record  = printmap[i].func_record;
+					print_prolog  = printmap[i].func_prolog;
+					print_epilog  = printmap[i].func_epilog;
 				}
 				break;
 			}
@@ -1127,10 +1060,6 @@ char 		Ident[IDENTLEN];
 		exit(255);
 	}
 
-	// this is the only case, where headers are printed.
-	if ( strncasecmp(print_format, "raw", 16) == 0 )
-		print_header = format_file_block_header;
-	
 	if ( aggregate && (flow_stat || element_stat) ) {
 		aggregate = 0;
 		LogError("Command line switch -s overwrites -a\n");
@@ -1174,7 +1103,7 @@ char 		Ident[IDENTLEN];
 
 	if ( fdump ) {
 		printf("StartNode: %i Engine: %s\n", Engine->StartNode, Engine->Extended ? "Extended" : "Fast");
-		DumpList(Engine);
+		DumpEngine(Engine);
 		exit(0);
 	}
 
@@ -1200,24 +1129,16 @@ char 		Ident[IDENTLEN];
 	}
 
 
-	if ( !(flow_stat || element_stat || wfile || quiet ) && record_header ) {
-		if ( user_format ) {
-			printf("%s\n", record_header);
-		} else {
-			// static format - no static format with header any more, but keep code anyway
-			if ( Getv6Mode() ) {
-				printf("%s\n", record_header);
-			} else
-				printf("%s\n", record_header);
-		}
+	if ( !(flow_stat || element_stat || wfile || quiet ) && print_prolog ) {
+		print_prolog();
 	}
 
 	nfprof_start(&profile_data);
 	sum_stat = process_data(wfile, element_stat, aggregate || flow_stat, print_order != NULL,
-						print_header, print_record, t_start, t_end, 
-						limitflows, do_tag, compress, do_xstat);
-	nfprof_end(&profile_data, total_flows);
-
+						print_record, t_start, t_end, 
+						limitRecords, do_tag, compress);
+	nfprof_end(&profile_data, recordCount);
+	
 	if ( total_bytes == 0 ) {
 		printf("No matched flows\n");
 		exit(0);
@@ -1228,7 +1149,7 @@ char 		Ident[IDENTLEN];
 			nffile_t *nffile = OpenNewFile(wfile, NULL, compress, is_anonymized, NULL);
 			if ( !nffile ) 
 				exit(255);
-			if ( ExportFlowTable(nffile, aggregate, bidir, date_sorted, extension_map_list) ) {
+			if ( ExportFlowTable(nffile, aggregate, bidir, GuessDir, date_sorted, extension_map_list) ) {
 				CloseUpdateFile(nffile, Ident );	
 			} else {
 				CloseFile(nffile);
@@ -1241,23 +1162,26 @@ char 		Ident[IDENTLEN];
 	}
 
 	if (flow_stat) {
-		PrintFlowStat(record_header, print_record, topN, do_tag, quiet, csv_output, extension_map_list);
+		PrintFlowStat(print_prolog, print_record, topN, do_tag, quiet, csv_output, extension_map_list);
 #ifdef DEVEL
 		printf("Loopcnt: %u\n", loopcnt);
 #endif
 	} 
 
 	if (element_stat) {
-		PrintElementStat(&sum_stat, plain_numbers, record_header, print_record, topN, do_tag, quiet, pipe_output, csv_output);
+		PrintElementStat(&sum_stat, printPlain, print_record, topN, do_tag, quiet, pipe_output, csv_output);
 	} 
 
-	if ( !quiet ) {
+	if ( print_epilog ) {
+		print_epilog();
+	}
+	if ( !quiet && !json_output ) {
 		if ( csv_output ) {
-			PrintSummary(&sum_stat, plain_numbers, csv_output);
+			PrintSummary(&sum_stat, printPlain, csv_output);
 		} else if ( !wfile ) {
 			if (is_anonymized)
 				printf("IP addresses anonymised\n");
-			PrintSummary(&sum_stat, plain_numbers, csv_output);
+			PrintSummary(&sum_stat, printPlain, csv_output);
 			if ( t_last_flow == 0 ) {
 				// in case of a pre 1.6.6 collected and empty flow file
  				printf("Time window: <unknown>\n");
@@ -1265,7 +1189,7 @@ char 		Ident[IDENTLEN];
  				printf("Time window: %s\n", TimeString(t_first_flow, t_last_flow));
 			}
 			printf("Total flows processed: %u, Blocks skipped: %u, Bytes read: %llu\n", 
-				total_flows, skipped_blocks, (unsigned long long)total_bytes);
+				recordCount, skipped_blocks, (unsigned long long)total_bytes);
 			nfprof_print(&profile_data, stdout);
 		}
 	}
