@@ -1,4 +1,6 @@
 /*
+ *  Copyright (c) 2017, Peter Haag
+ *  Copyright (c) 2016, Peter Haag
  *  Copyright (c) 2014, Peter Haag
  *  Copyright (c) 2009, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
@@ -28,29 +30,14 @@
  *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
  *  POSSIBILITY OF SUCH DAMAGE.
  *  
- *  $Author: haag $
- *
- *  $Id: nffile_inline.c 40 2009-12-16 10:41:44Z haag $
- *
- *  $LastChangedRevision: 40 $
- *	
  */
 
-/* 
- * nffile_inline.c is needed for daemon code as well as normal stdio code 
- * therefore a generic LogError is defined, which maps to the 
- * approriate logging channel - either stderr or syslog
- */
-void LogError(char *format, ...);
-
 static inline int CheckBufferSpace(nffile_t *nffile, size_t required);
 
 static inline void AppendToBuffer(nffile_t *nffile, void *record, size_t required);
 
 static inline void CopyV6IP(uint32_t *dst, uint32_t *src);
 
-static inline void ConvertCommonV0(void *record, common_record_t *flow_record);
-
 static inline void ExpandRecord_v2(common_record_t *input_record, extension_info_t *extension_info, exporter_info_record_t *exporter_info, master_record_t *output_record );
 
 #ifdef NEED_PACKRECORD
@@ -59,9 +46,7 @@ static void PackRecord(master_record_t *master_record, nffile_t *nffile);
 
 static inline int CheckBufferSpace(nffile_t *nffile, size_t required) {
 
-#ifdef DEVEL
-//	printf("Buffer Size %u\n", nffile->block_header->size);
-#endif
+	dbg_printf("Buffer Size %u\n", nffile->block_header->size);
 	// flush current buffer to disc
 	if ( (nffile->block_header->size + required )  > WRITE_BUFFSIZE ) {
 
@@ -88,22 +73,6 @@ static inline void CopyV6IP(uint32_t *dst, uint32_t *src) {
 	dst[3] = src[3];
 } // End of CopyV6IP
 
-static inline void ConvertCommonV0(void *record, common_record_t *flow_record) {
-common_record_v0_t *flow_record_v0 = (common_record_v0_t *)record;
-
-	// copy v0 common record
-	memcpy((void *)flow_record, record, COMMON_RECORDV0_DATA_SIZE);
-	memcpy((void *)flow_record->data, (void *)flow_record_v0->data, flow_record_v0->size - COMMON_RECORDV0_DATA_SIZE);
-
-	// fix record differences
-	flow_record->type			= CommonRecordType;
-	flow_record->size			+= (COMMON_RECORD_DATA_SIZE - COMMON_RECORDV0_DATA_SIZE);
-	flow_record->flags			= flow_record_v0->flags;
-	flow_record->exporter_sysid = flow_record_v0->exporter_sysid;
-	flow_record->reserved 		= 0;
-
-} // End of ConvertCommonV0
-
 /*
  * Expand file record into master record for further processing
  * LP64 CPUs need special 32bit operations as it is not guarateed, that 64bit
@@ -140,6 +109,7 @@ void		*p = (void *)input_record;
 	} else {
 		output_record->exp_ref 		  = NULL;
 	}
+	output_record->label = NULL;
 
 	// map icmp type/code in it's own vars
 	output_record->icmp = output_record->dstport;
@@ -148,19 +118,19 @@ void		*p = (void *)input_record;
 	if ( (input_record->flags & FLAG_IPV6_ADDR) != 0 )	{ // IPv6
 		// IPv6
 		// keep compiler happy
-		// memcpy((void *)output_record->v6.srcaddr, p, 4 * sizeof(uint64_t));	
+		// memcpy((void *)output_record->V6.srcaddr, p, 4 * sizeof(uint64_t));	
 		memcpy((void *)output_record->ip_union._ip_64.addr, p, 4 * sizeof(uint64_t));	
 		p = (void *)((pointer_addr_t)p + 4 * sizeof(uint64_t));
 	} else { 	
 		// IPv4
 		u = (uint32_t *)p;
-		output_record->v6.srcaddr[0] = 0;
-		output_record->v6.srcaddr[1] = 0;
-		output_record->v4.srcaddr 	 = u[0];
+		output_record->V6.srcaddr[0] = 0;
+		output_record->V6.srcaddr[1] = 0;
+		output_record->V4.srcaddr 	 = u[0];
 
-		output_record->v6.dstaddr[0] = 0;
-		output_record->v6.dstaddr[1] = 0;
-		output_record->v4.dstaddr 	 = u[1];
+		output_record->V6.dstaddr[0] = 0;
+		output_record->V6.dstaddr[1] = 0;
+		output_record->V4.dstaddr 	 = u[1];
 		p = (void *)((pointer_addr_t)p + 2 * sizeof(uint32_t));
 	}
 
@@ -237,29 +207,29 @@ void		*p = (void *)input_record;
 				} break;
 			case EX_NEXT_HOP_v4: {
 				tpl_ext_9_t *tpl = (tpl_ext_9_t *)p;
-				output_record->ip_nexthop.v6[0] = 0;
-				output_record->ip_nexthop.v6[1] = 0;
-				output_record->ip_nexthop.v4	= tpl->nexthop;
+				output_record->ip_nexthop.V6[0] = 0;
+				output_record->ip_nexthop.V6[1] = 0;
+				output_record->ip_nexthop.V4	= tpl->nexthop;
 				p = (void *)tpl->data;
 				ClearFlag(output_record->flags, FLAG_IPV6_NH);
 				} break;
 			case EX_NEXT_HOP_v6: {
 				tpl_ext_10_t *tpl = (tpl_ext_10_t *)p;
-				CopyV6IP((uint32_t *)output_record->ip_nexthop.v6, (uint32_t *)tpl->nexthop);
+				CopyV6IP((uint32_t *)output_record->ip_nexthop.V6, (uint32_t *)tpl->nexthop);
 				p = (void *)tpl->data;
 				SetFlag(output_record->flags, FLAG_IPV6_NH);
 				} break;
 			case EX_NEXT_HOP_BGP_v4: {
 				tpl_ext_11_t *tpl = (tpl_ext_11_t *)p;
-				output_record->bgp_nexthop.v6[0] = 0;
-				output_record->bgp_nexthop.v6[1] = 0;
-				output_record->bgp_nexthop.v4	= tpl->bgp_nexthop;
+				output_record->bgp_nexthop.V6[0] = 0;
+				output_record->bgp_nexthop.V6[1] = 0;
+				output_record->bgp_nexthop.V4	= tpl->bgp_nexthop;
 				ClearFlag(output_record->flags, FLAG_IPV6_NHB);
 				p = (void *)tpl->data;
 				} break;
 			case EX_NEXT_HOP_BGP_v6: {
 				tpl_ext_12_t *tpl = (tpl_ext_12_t *)p;
-				CopyV6IP((uint32_t *)output_record->bgp_nexthop.v6, (uint32_t *)tpl->bgp_nexthop);
+				CopyV6IP((uint32_t *)output_record->bgp_nexthop.V6, (uint32_t *)tpl->bgp_nexthop);
 				p = (void *)tpl->data;
 				SetFlag(output_record->flags, FLAG_IPV6_NHB);
 				} break;
@@ -336,15 +306,15 @@ void		*p = (void *)input_record;
 			} break;
 			case EX_ROUTER_IP_v4: {
 				tpl_ext_23_t *tpl = (tpl_ext_23_t *)p;
-				output_record->ip_router.v6[0] = 0;
-				output_record->ip_router.v6[1] = 0;
-				output_record->ip_router.v4	= tpl->router_ip;
+				output_record->ip_router.V6[0] = 0;
+				output_record->ip_router.V6[1] = 0;
+				output_record->ip_router.V4	= tpl->router_ip;
 				p = (void *)tpl->data;
 				ClearFlag(output_record->flags, FLAG_IPV6_EXP);
 				} break;
 			case EX_ROUTER_IP_v6: {
 				tpl_ext_24_t *tpl = (tpl_ext_24_t *)p;
-				CopyV6IP((uint32_t *)output_record->ip_router.v6, (uint32_t *)tpl->router_ip);
+				CopyV6IP((uint32_t *)output_record->ip_router.V6, (uint32_t *)tpl->router_ip);
 				p = (void *)tpl->data;
 				SetFlag(output_record->flags, FLAG_IPV6_EXP);
 				} break;
@@ -387,6 +357,7 @@ void		*p = (void *)input_record;
 				output_record->event_flag = FW_EVENT;
 				output_record->fw_xevent  = tpl->fw_xevent;
 				output_record->icmp = tpl->nsel_icmp;
+				output_record->sec_group_tag = tpl->sec_group_tag;
 				p = (void *)tpl->data;
 			} break;
 			case EX_NSEL_XLATE_PORTS: {
@@ -397,21 +368,21 @@ void		*p = (void *)input_record;
 			} break;
 			case EX_NSEL_XLATE_IP_v4: {
 				tpl_ext_39_t *tpl = (tpl_ext_39_t *)p;
-				output_record->xlate_src_ip.v6[0] = 0;
-				output_record->xlate_src_ip.v6[1] = 0;
-				output_record->xlate_src_ip.v4	= tpl->xlate_src_ip;
-				output_record->xlate_dst_ip.v6[0] = 0;
-				output_record->xlate_dst_ip.v6[1] = 0;
-				output_record->xlate_dst_ip.v4	= tpl->xlate_dst_ip;
+				output_record->xlate_src_ip.V6[0] = 0;
+				output_record->xlate_src_ip.V6[1] = 0;
+				output_record->xlate_src_ip.V4	= tpl->xlate_src_ip;
+				output_record->xlate_dst_ip.V6[0] = 0;
+				output_record->xlate_dst_ip.V6[1] = 0;
+				output_record->xlate_dst_ip.V4	= tpl->xlate_dst_ip;
 				p = (void *)tpl->data;
 				output_record->xlate_flags = 0;
 				} break;
 			case EX_NSEL_XLATE_IP_v6: {
 				tpl_ext_40_t *tpl = (tpl_ext_40_t *)p;
-				output_record->xlate_src_ip.v6[0] = tpl->xlate_src_ip[0];
-				output_record->xlate_src_ip.v6[1] = tpl->xlate_src_ip[1];
-				output_record->xlate_dst_ip.v6[0] = tpl->xlate_dst_ip[0];
-				output_record->xlate_dst_ip.v6[1] = tpl->xlate_dst_ip[1];
+				output_record->xlate_src_ip.V6[0] = tpl->xlate_src_ip[0];
+				output_record->xlate_src_ip.V6[1] = tpl->xlate_src_ip[1];
+				output_record->xlate_dst_ip.V6[0] = tpl->xlate_dst_ip[0];
+				output_record->xlate_dst_ip.V6[1] = tpl->xlate_dst_ip[1];
 				p = (void *)tpl->data;
 				output_record->xlate_flags = 1;
 				} break;
@@ -456,12 +427,12 @@ void		*p = (void *)input_record;
 			// compat record v1.6.10
 			case EX_NEL_GLOBAL_IP_v4: {
 				tpl_ext_47_t *tpl = (tpl_ext_47_t *)p;
-				output_record->xlate_src_ip.v6[0] = 0;
-				output_record->xlate_src_ip.v6[1] = 0;
-				output_record->xlate_src_ip.v4	= tpl->nat_inside;
-				output_record->xlate_dst_ip.v6[0] = 0;
-				output_record->xlate_dst_ip.v6[1] = 0;
-				output_record->xlate_dst_ip.v4	= tpl->nat_outside;
+				output_record->xlate_src_ip.V6[0] = 0;
+				output_record->xlate_src_ip.V6[1] = 0;
+				output_record->xlate_src_ip.V4	= tpl->nat_inside;
+				output_record->xlate_dst_ip.V6[0] = 0;
+				output_record->xlate_dst_ip.V6[1] = 0;
+				output_record->xlate_dst_ip.V4	= tpl->nat_outside;
 				p = (void *)tpl->data;
 
 				output_record->xlate_src_port = compat_nel_bug.port[0];
@@ -536,14 +507,14 @@ int		i;
 	if ( (master_record->flags & FLAG_IPV6_ADDR) != 0 )	{ // IPv6
 		// IPv6
 		// keep compiler happy
-		// memcpy(p, (void *)master_record->v6.srcaddr, 4 * sizeof(uint64_t));	
+		// memcpy(p, (void *)master_record->V6.srcaddr, 4 * sizeof(uint64_t));	
 		memcpy(p, (void *)master_record->ip_union._ip_64.addr, 4 * sizeof(uint64_t));	
 		p = (void *)((pointer_addr_t)p + 4 * sizeof(uint64_t));
 	} else { 	
 		// IPv4
 		uint32_t *u = (uint32_t *)p;
-		u[0] = master_record->v4.srcaddr;
-		u[1] = master_record->v4.dstaddr;
+		u[0] = master_record->V4.srcaddr;
+		u[1] = master_record->V4.dstaddr;
 		p = (void *)((pointer_addr_t)p + 2 * sizeof(uint32_t));
 	}
 
@@ -617,24 +588,24 @@ int		i;
 				} break;
 			case EX_NEXT_HOP_v4: {
 				tpl_ext_9_t *tpl = (tpl_ext_9_t *)p;
-				tpl->nexthop = master_record->ip_nexthop.v4;
+				tpl->nexthop = master_record->ip_nexthop.V4;
 				p = (void *)tpl->data;
 				} break;
 			case EX_NEXT_HOP_v6: {
 				tpl_ext_10_t *tpl = (tpl_ext_10_t *)p;
-				tpl->nexthop[0] = master_record->ip_nexthop.v6[0];
-				tpl->nexthop[1] = master_record->ip_nexthop.v6[1];
+				tpl->nexthop[0] = master_record->ip_nexthop.V6[0];
+				tpl->nexthop[1] = master_record->ip_nexthop.V6[1];
 				p = (void *)tpl->data;
 				} break;
 			case EX_NEXT_HOP_BGP_v4: {
 				tpl_ext_11_t *tpl = (tpl_ext_11_t *)p;
-				tpl->bgp_nexthop = master_record->bgp_nexthop.v4;
+				tpl->bgp_nexthop = master_record->bgp_nexthop.V4;
 				p = (void *)tpl->data;
 				} break;
 			case EX_NEXT_HOP_BGP_v6: {
 				tpl_ext_12_t *tpl = (tpl_ext_12_t *)p;
-				tpl->bgp_nexthop[0] = master_record->bgp_nexthop.v6[0];
-				tpl->bgp_nexthop[1] = master_record->bgp_nexthop.v6[1];
+				tpl->bgp_nexthop[0] = master_record->bgp_nexthop.V6[0];
+				tpl->bgp_nexthop[1] = master_record->bgp_nexthop.V6[1];
 				p = (void *)tpl->data;
 				} break;
 			case EX_VLAN: {
@@ -709,13 +680,13 @@ int		i;
 				} break;
 			case EX_ROUTER_IP_v4: {
 				tpl_ext_23_t *tpl = (tpl_ext_23_t *)p;
-				tpl->router_ip = master_record->ip_router.v4;
+				tpl->router_ip = master_record->ip_router.V4;
 				p = (void *)tpl->data;
 				} break;
 			case EX_ROUTER_IP_v6: {
 				tpl_ext_24_t *tpl = (tpl_ext_24_t *)p;
-				tpl->router_ip[0] = master_record->ip_router.v6[0];
-				tpl->router_ip[1] = master_record->ip_router.v6[1];
+				tpl->router_ip[0] = master_record->ip_router.V6[0];
+				tpl->router_ip[1] = master_record->ip_router.V6[1];
 				p = (void *)tpl->data;
 				} break;
 			case EX_ROUTER_ID: {
@@ -730,18 +701,18 @@ int		i;
 				tpl->bgpPrevAdjacentAS = master_record->bgpPrevAdjacentAS;
 				p = (void *)tpl->data;
 				} break;
-			case EX_LATENCY: {
-				tpl_ext_latency_t *tpl = (tpl_ext_latency_t *)p;
-				tpl->client_nw_delay_usec = master_record->client_nw_delay_usec;
-				tpl->server_nw_delay_usec = master_record->server_nw_delay_usec;
-				tpl->appl_latency_usec = master_record->appl_latency_usec;
-				p = (void *)tpl->data;
-				} break;
 			case EX_RECEIVED: {
 				tpl_ext_27_t *tpl = (tpl_ext_27_t *)p;
 				tpl->received = master_record->received;
 				p = (void *)tpl->data;
 				} break;
+			case EX_LATENCY: {
+				tpl_ext_latency_t *tpl = (tpl_ext_latency_t *)p;
+				tpl->client_nw_delay_usec = master_record->client_nw_delay_usec;
+				tpl->server_nw_delay_usec = master_record->server_nw_delay_usec;
+				tpl->appl_latency_usec	  = master_record->appl_latency_usec;
+				p = (void *)tpl->data;
+			} break;
 #ifdef NSEL
 			case EX_NSEL_COMMON: {
 				tpl_ext_37_t *tpl = (tpl_ext_37_t *)p;
@@ -750,7 +721,7 @@ int		i;
 				tpl->fw_event   = master_record->event;
 				tpl->nsel_icmp  = master_record->icmp;
 				tpl->fill  = 0;
-				tpl->fill2 = 0;
+				tpl->sec_group_tag = master_record->sec_group_tag;
 				tpl->fw_xevent = master_record->fw_xevent;
 				p = (void *)tpl->data;
 				} break;
@@ -762,17 +733,17 @@ int		i;
 				} break;
 			case EX_NSEL_XLATE_IP_v4: {
 				tpl_ext_39_t *tpl = (tpl_ext_39_t *)p;
-				tpl->xlate_src_ip = master_record->xlate_src_ip.v4;
-				tpl->xlate_dst_ip = master_record->xlate_dst_ip.v4;
+				tpl->xlate_src_ip = master_record->xlate_src_ip.V4;
+				tpl->xlate_dst_ip = master_record->xlate_dst_ip.V4;
 				p = (void *)tpl->data;
 				} break;
 			case EX_NSEL_XLATE_IP_v6: {
 				tpl_ext_40_t *tpl = (tpl_ext_40_t *)p;
-				tpl->xlate_src_ip[0] = master_record->xlate_src_ip.v6[0];
-				tpl->xlate_src_ip[1] = master_record->xlate_src_ip.v6[1];
+				tpl->xlate_src_ip[0] = master_record->xlate_src_ip.V6[0];
+				tpl->xlate_src_ip[1] = master_record->xlate_src_ip.V6[1];
 				p = (void *)tpl->data;
-				tpl->xlate_dst_ip[0] = master_record->xlate_dst_ip.v6[0];
-				tpl->xlate_dst_ip[1] = master_record->xlate_dst_ip.v6[1];
+				tpl->xlate_dst_ip[0] = master_record->xlate_dst_ip.V6[0];
+				tpl->xlate_dst_ip[1] = master_record->xlate_dst_ip.V6[1];
 				p = (void *)tpl->data;
 				} break;
 			case EX_NSEL_ACL: {
@@ -817,7 +788,7 @@ int		i;
 		}
 	}
 
-	nffile->block_header->size 		+= required;
+	nffile->block_header->size += required;
 	nffile->block_header->NumRecords++;
 #ifdef DEVEL
 	if ( ((pointer_addr_t)p - (pointer_addr_t)nffile->buff_ptr) != required ) {
