@@ -1,6 +1,5 @@
 /*
- *  Copyright (c) 2014, Peter Haag
- *  Copyright (c) 2009, Peter Haag
+ *  Copyright (c) 2009-2020 Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *  
@@ -28,12 +27,6 @@
  *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
  *  POSSIBILITY OF SUCH DAMAGE.
  *  
- *  $Author: haag $
- *
- *  $Id: nfgen.c 39 2009-11-25 08:11:15Z haag $
- *
- *  $LastChangedRevision: 39 $
- *	
  */
 
 #include "config.h"
@@ -56,16 +49,11 @@
 #include <stdint.h>
 #endif
 
-#include "nffile.h"
-#include "nfx.h"
-#include "nfnet.h"
-#include "nf_common.h"
 #include "util.h"
-#include "bookkeeper.h"
-#include "nfxstat.h"
+#include "nfdump.h"
+#include "nffile.h"
 #include "collector.h"
-#include "exporter.h"
-#include "netflow_v5_v7.h"
+#include "nfx.h"
 
 extern extension_descriptor_t extension_descriptor[];
 
@@ -96,18 +84,18 @@ static void SetIPaddress(master_record_t *record, int af,  char *src_ip, char *d
 
 	if ( af == PF_INET6 ) {
 		SetFlag(record->flags, FLAG_IPV6_ADDR);
-		inet_pton(PF_INET6, src_ip, &(record->v6.srcaddr[0]));
-		inet_pton(PF_INET6, dst_ip, &(record->v6.dstaddr[0]));
-		record->v6.srcaddr[0] = ntohll(record->v6.srcaddr[0]);
-		record->v6.srcaddr[1] = ntohll(record->v6.srcaddr[1]);
-		record->v6.dstaddr[0] = ntohll(record->v6.dstaddr[0]);
-		record->v6.dstaddr[1] = ntohll(record->v6.dstaddr[1]);
+		inet_pton(PF_INET6, src_ip, &(record->V6.srcaddr[0]));
+		inet_pton(PF_INET6, dst_ip, &(record->V6.dstaddr[0]));
+		record->V6.srcaddr[0] = ntohll(record->V6.srcaddr[0]);
+		record->V6.srcaddr[1] = ntohll(record->V6.srcaddr[1]);
+		record->V6.dstaddr[0] = ntohll(record->V6.dstaddr[0]);
+		record->V6.dstaddr[1] = ntohll(record->V6.dstaddr[1]);
 	} else {
 		ClearFlag(record->flags, FLAG_IPV6_ADDR);
-		inet_pton(PF_INET, src_ip, &record->v4.srcaddr);
-		inet_pton(PF_INET, dst_ip, &record->v4.dstaddr);
-		record->v4.srcaddr = ntohl(record->v4.srcaddr);
-		record->v4.dstaddr = ntohl(record->v4.dstaddr);
+		inet_pton(PF_INET, src_ip, &record->V4.srcaddr);
+		inet_pton(PF_INET, dst_ip, &record->V4.dstaddr);
+		record->V4.srcaddr = ntohl(record->V4.srcaddr);
+		record->V4.dstaddr = ntohl(record->V4.dstaddr);
 	}
 
 } // End of SetIPaddress
@@ -116,13 +104,13 @@ static void SetNextIPaddress(master_record_t *record, int af,  char *next_ip) {
 
 	if ( af == PF_INET6 ) {
 		SetFlag(record->flags, FLAG_IPV6_NH);
-		inet_pton(PF_INET6, next_ip, &(record->ip_nexthop.v6[0]));
-		record->ip_nexthop.v6[0] = ntohll(record->ip_nexthop.v6[0]);
-		record->ip_nexthop.v6[1] = ntohll(record->ip_nexthop.v6[1]);
+		inet_pton(PF_INET6, next_ip, &(record->ip_nexthop.V6[0]));
+		record->ip_nexthop.V6[0] = ntohll(record->ip_nexthop.V6[0]);
+		record->ip_nexthop.V6[1] = ntohll(record->ip_nexthop.V6[1]);
 	} else {
 		ClearFlag(record->flags, FLAG_IPV6_NH);
-		inet_pton(PF_INET, next_ip, &record->ip_nexthop.v4);
-		record->ip_nexthop.v4 = ntohl(record->ip_nexthop.v4);
+		inet_pton(PF_INET, next_ip, &record->ip_nexthop.V4);
+		record->ip_nexthop.V4 = ntohl(record->ip_nexthop.V4);
 	}
 
 } // End of SetNextIPaddress
@@ -131,13 +119,13 @@ static void SetRouterIPaddress(master_record_t *record, int af,  char *next_ip)
 
 	if ( af == PF_INET6 ) {
 		SetFlag(record->flags, FLAG_IPV6_NH);
-		inet_pton(PF_INET6, next_ip, &(record->ip_router.v6[0]));
-		record->ip_router.v6[0] = ntohll(record->ip_router.v6[0]);
-		record->ip_router.v6[1] = ntohll(record->ip_router.v6[1]);
+		inet_pton(PF_INET6, next_ip, &(record->ip_router.V6[0]));
+		record->ip_router.V6[0] = ntohll(record->ip_router.V6[0]);
+		record->ip_router.V6[1] = ntohll(record->ip_router.V6[1]);
 	} else {
 		ClearFlag(record->flags, FLAG_IPV6_NH);
-		inet_pton(PF_INET, next_ip, &record->ip_router.v4);
-		record->ip_router.v4 = ntohl(record->ip_router.v4);
+		inet_pton(PF_INET, next_ip, &record->ip_router.V4);
+		record->ip_router.V4 = ntohl(record->ip_router.V4);
 	}
 
 } // End of SetRouterIPaddress
@@ -146,13 +134,13 @@ static void SetBGPNextIPaddress(master_record_t *record, int af,  char *next_ip)
 
 	if ( af == PF_INET6 ) {
 		SetFlag(record->flags, FLAG_IPV6_NHB);
-		inet_pton(PF_INET6, next_ip, &(record->bgp_nexthop.v6[0]));
-		record->bgp_nexthop.v6[0] = ntohll(record->bgp_nexthop.v6[0]);
-		record->bgp_nexthop.v6[1] = ntohll(record->bgp_nexthop.v6[1]);
+		inet_pton(PF_INET6, next_ip, &(record->bgp_nexthop.V6[0]));
+		record->bgp_nexthop.V6[0] = ntohll(record->bgp_nexthop.V6[0]);
+		record->bgp_nexthop.V6[1] = ntohll(record->bgp_nexthop.V6[1]);
 	} else {
 		ClearFlag(record->flags, FLAG_IPV6_NHB);
-		inet_pton(PF_INET, next_ip, &record->bgp_nexthop.v4);
-		record->bgp_nexthop.v4 = ntohl(record->bgp_nexthop.v4);
+		inet_pton(PF_INET, next_ip, &record->bgp_nexthop.V4);
+		record->bgp_nexthop.V4 = ntohl(record->bgp_nexthop.V4);
 	}
 
 } // End of SetBGPNextIPaddress
@@ -232,7 +220,7 @@ nffile_t			*nffile;
 	}
 	memset((void *)&record, 0, sizeof(record));
 
-	nffile = OpenNewFile("-", NULL, 0, 0, NULL);
+	nffile = OpenNewFile("-", NULL, NOT_COMPRESSED, 0, NULL);
 	if ( !nffile ) {
 		exit(255);
 	}
@@ -400,7 +388,7 @@ nffile_t			*nffile;
 	SetIPaddress(&record,  PF_INET, "172.16.13.66", "192.168.170.112");
 	record.srcport 	 	= 0;
 	record.dstport 	 	= 8;
-	record.prot 	 	= 1;
+	record.prot 	 	= IPPROTO_ICMP;
 	record.tcp_flags 	= 0;
 	record.tos 		 	= 0;
 	record.dPkts 	 	= 50002;
@@ -431,7 +419,9 @@ nffile_t			*nffile;
 	UpdateRecord(&record);
 	PackRecord(&record, nffile);
 
-	SetIPaddress(&record,  PF_INET6, "2001:234:aabb::211:24ff:fe80:d01e", "2001:620::8:203:baff:fe52:38e5");
+	SetIPaddress(&record,  PF_INET6, "2001:234:aabb:cc:211:24ff:fe80:d01e", "2001:620:1f:8:203:baff:fe52:38e5");
+	record.src_mask		= 88;
+	record.dst_mask		= 48;
 	record.srcport 	 = 10240;
 	record.dstport 	 = 52345;
 	record.dPkts 	 = 10100;
@@ -460,6 +450,8 @@ nffile_t			*nffile;
 	SetIPaddress(&record,  PF_INET, "172.16.14.18", "192.168.170.113");
 //	SetNextIPaddress(&record,  PF_INET, "172.72.1.2");
 //	SetBGPNextIPaddress(&record,  PF_INET, "172.73.2.3");
+	record.src_mask		= 16;
+	record.dst_mask		= 24;
 	record.srcport 	 = 10240;
 	record.dstport 	 = 52345;
 	record.dPkts 	 = 10100000;
