@@ -1,40 +1,36 @@
 /*
- *  Copyright (c) 2014, Peter Haag
+ *  Copyright (c) 2014-2020, Peter Haag
  *  All rights reserved.
- *  
- *  Redistribution and use in source and binary forms, with or without 
+ *
+ *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
- *  
- *   * Redistributions of source code must retain the above copyright notice, 
+ *
+ *   * Redistributions of source code must retain the above copyright notice,
  *	 this list of conditions and the following disclaimer.
- *   * Redistributions in binary form must reproduce the above copyright notice, 
- *	 this list of conditions and the following disclaimer in the documentation 
+ *   * Redistributions in binary form must reproduce the above copyright notice,
+ *	 this list of conditions and the following disclaimer in the documentation
  *	 and/or other materials provided with the distribution.
- *   * Neither the name of the author nor the names of its contributors may be 
- *	 used to endorse or promote products derived from this software without 
+ *   * Neither the name of the author nor the names of its contributors may be
+ *	 used to endorse or promote products derived from this software without
  *	 specific prior written permission.
- *  
- *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
- *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
- *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
- *  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
- *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
- *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
- *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
- *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
- *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
- *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
+ *
+ *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ *  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
+ *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  *  POSSIBILITY OF SUCH DAMAGE.
- *  
- *  $Author$
  *
- *  $Id$
- *
- *  $LastChangedRevision$
- *  
  */
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
 
 #ifdef HAVE_FEATURES_H
 #include <features.h>
@@ -68,9 +64,9 @@
 #include <pcap.h>
 
 #include "util.h"
+#include "nfdump.h"
 #include "nffile.h"
 #include "bookkeeper.h"
-#include "nfxstat.h"
 #include "collector.h"
 #include "flowtree.h"
 #include "ipfrag.h"
@@ -78,29 +74,15 @@
 #include "content_dns.h"
 #include "netflow_pcap.h"
 
-#ifndef DEVEL
-#   define dbg_printf(...) /* printf(__VA_ARGS__) */
-#else
-#   define dbg_printf(...) printf(__VA_ARGS__)
-#endif
-
-static inline void ProcessTCPFlow(FlowSource_t	*fs, struct FlowNode *NewNode );
-
-static inline void ProcessUDPFlow(FlowSource_t	*fs, struct FlowNode *NewNode );
-
-static inline void ProcessICMPFlow(FlowSource_t	*fs, struct FlowNode *NewNode );
-
-static inline void ProcessOtherFlow(FlowSource_t	*fs, struct FlowNode *NewNode );
-
 struct pcap_timeval {
-	int32_t tv_sec;	   /* seconds */
-	int32_t tv_usec;	  /* microseconds */
+	int32_t tv_sec;   /* seconds */
+	int32_t tv_usec;  /* microseconds */
 };
 
 struct pcap_sf_pkthdr {
 	struct pcap_timeval ts; /* time stamp */
-	uint32_t	caplen;	 /* length of portion present */
-	uint32_t	len;		/* length this packet (off wire) */
+	uint32_t	caplen;	    /* length of portion present */
+	uint32_t	len;        /* length this packet (off wire) */
 };
 
 typedef struct vlan_hdr_s {
@@ -113,7 +95,13 @@ typedef struct gre_hdr_s {
   uint16_t type;
 } gre_hdr_t;
 
-int lock_sync = 0;
+static inline void ProcessTCPFlow(FlowSource_t	*fs, struct FlowNode *NewNode );
+
+static inline void ProcessUDPFlow(FlowSource_t	*fs, struct FlowNode *NewNode );
+
+static inline void ProcessICMPFlow(FlowSource_t	*fs, struct FlowNode *NewNode );
+
+static inline void ProcessOtherFlow(FlowSource_t	*fs, struct FlowNode *NewNode );
 
 pcapfile_t *OpenNewPcapFile(pcap_t *p, char *filename, pcapfile_t *pcapfile) {
 
@@ -121,7 +109,7 @@ pcapfile_t *OpenNewPcapFile(pcap_t *p, char *filename, pcapfile_t *pcapfile) {
 		// Create struct
 		pcapfile = calloc(1, sizeof(pcapfile_t));
 		if ( !pcapfile ) {
-			LogError("malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
+			LogError("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno) );
 			return NULL;
 		}
 		pthread_mutex_init(&pcapfile->m_pbuff, NULL);
@@ -130,14 +118,14 @@ pcapfile_t *OpenNewPcapFile(pcap_t *p, char *filename, pcapfile_t *pcapfile) {
 		pcapfile->data_buffer = malloc(BUFFSIZE);
 		if ( !pcapfile->data_buffer ) {
 			free(pcapfile);
-			LogError("malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
+			LogError("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno) );
 			return NULL;
 		}
 		pcapfile->alternate_buffer = malloc(BUFFSIZE);
 		if ( !pcapfile->data_buffer ) {
 			free(pcapfile->data_buffer);
 			free(pcapfile);
-			LogError("malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
+			LogError("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno) );
 			return NULL;
 		}
 		pcapfile->data_ptr		 = pcapfile->data_buffer;
@@ -147,16 +135,21 @@ pcapfile_t *OpenNewPcapFile(pcap_t *p, char *filename, pcapfile_t *pcapfile) {
 	}
 
 	if ( filename ) {
-		pcapfile->pd = pcap_dump_open(p, filename);
+		FILE* pFile = fopen(filename, "wb"); 
+		if ( !pFile ) {
+			LogError("fopen() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno) );
+			return NULL;
+		}
+		pcapfile->pd = pcap_dump_fopen(p, pFile);
 		if ( !pcapfile->pd ) {
 			LogError("Fatal: pcap_dump_open() failed for file '%s': %s", filename, pcap_geterr(p));
 			return NULL;
 		} else {
-			fflush((FILE *)pcapfile->pd);
-			pcapfile->pfd = fileno((FILE *)pcapfile->pd);
+			fflush(pFile);
+			pcapfile->pfd = fileno((FILE *)pFile);
 			return pcapfile;
 		}
-	} else 
+	} else
 		return pcapfile;
 
 } // End of OpenNewPcapFile
@@ -164,10 +157,7 @@ pcapfile_t *OpenNewPcapFile(pcap_t *p, char *filename, pcapfile_t *pcapfile) {
 int ClosePcapFile(pcapfile_t *pcapfile) {
 int err = 0;
 
-	if ( fclose((FILE *)pcapfile->pd) < 0 ) {
-		LogError("close() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
-		err = errno;
-	}
+	pcap_dump_close(pcapfile->pd);
 	pcapfile->pfd = -1;
 
 	return err;
@@ -204,7 +194,7 @@ void *_b;
 		if( pcap_stats(pcapfile->p, &p_stat) < 0) {
 			LogError("pcap_stats() failed: %s", pcap_geterr(pcapfile->p));
 		} else {
-			LogInfo("Packets received: %u, dropped: %u, dropped by interface: %u ", 
+			LogInfo("Packets received: %u, dropped: %u, dropped by interface: %u ",
 				p_stat.ps_recv, p_stat.ps_drop, p_stat.ps_ifdrop );
 		}
 	}
@@ -260,13 +250,14 @@ size_t	size = sizeof(struct pcap_sf_pkthdr) + h->caplen;
 
 } // End of PcapDump
 
-static inline void ProcessTCPFlow(FlowSource_t	*fs, struct FlowNode *NewNode ) {
+static inline void ProcessTCPFlow(FlowSource_t *fs, struct FlowNode *NewNode ) {
 struct FlowNode *Node;
 
 	assert(NewNode->memflag == NODE_IN_USE);
 	Node = Insert_Node(NewNode);
-	// if insert fails, the existing node is returned -> flow exists already
+	// Return existing Node if flow exists already, otherwise insert es new
 	if ( Node == NULL ) {
+		// Insert as new
 		dbg_printf("New TCP flow: Packets: %u, Bytes: %u\n", NewNode->packets, NewNode->bytes);
 
 		// in case it's a FIN/RST only packet - immediately flush it
@@ -274,25 +265,30 @@ struct FlowNode *Node;
 			// flush node
 			if ( StorePcapFlow(fs, NewNode) ) {
 				Remove_Node(NewNode);
-			} 
+			}
 		}
 
-		if ( !CacheCheck() ) {
-			uint32_t NumFlows;
-			LogError("Node cache exhausted! - Immediate flush - increase flow cache!!");	
-			NumFlows  = Flush_FlowTree(fs);
-			LogError("Flushed flows: %u", NumFlows);	
+		if ( Link_RevNode(NewNode)) {
+			// if we could link this new node, it is the server answer
+			// -> calculate server latency
+			SetServer_latency(NewNode);
 		}
 		return;
 	}
 
 	assert(Node->memflag == NODE_IN_USE);
 
+	// check for first client ACK for client latency
+	if ( Node->latency.flag == 1 ) {
+		SetClient_latency(Node, &(NewNode->t_first));
+	} else if ( Node->latency.flag == 2 ) {
+		SetApplication_latency(Node, &(NewNode->t_first));
+	}
 	// update existing flow
 	Node->flags |= NewNode->flags;
 	Node->packets++;
-	Node->bytes += NewNode->bytes; 
-	Node->t_last = NewNode->t_last; 
+	Node->bytes += NewNode->bytes;
+	Node->t_last = NewNode->t_last;
 	dbg_printf("Existing TCP flow: Packets: %u, Bytes: %u\n", Node->packets, Node->bytes);
 
 	if ( NewNode->fin == FIN_NODE) {
@@ -300,11 +296,11 @@ struct FlowNode *Node;
 		Node->fin = FIN_NODE;
 		if ( StorePcapFlow(fs, Node) ) {
 			Remove_Node(Node);
-		} 
+		}
 	} else {
 		Free_Node(NewNode);
 	}
- 
+
 
 } // End of ProcessTCPFlow
 
@@ -312,7 +308,7 @@ static inline void ProcessUDPFlow(FlowSource_t	*fs, struct FlowNode *NewNode ) {
 struct FlowNode *Node;
 
 	assert(NewNode->memflag == NODE_IN_USE);
-	// Flush DNS queries directly 
+	// Flush DNS queries directly
 	if ( NewNode->src_port == 53 || NewNode->dst_port == 53 ) {
 		StorePcapFlow(fs, NewNode);
 		Free_Node(NewNode);
@@ -323,18 +319,15 @@ struct FlowNode *Node;
 	Node = Insert_Node(NewNode);
 	// if insert fails, the existing node is returned -> flow exists already
 	if ( Node == NULL ) {
-		AppendUDPNode(NewNode);
 		dbg_printf("New UDP flow: Packets: %u, Bytes: %u\n", NewNode->packets, NewNode->bytes);
 		return;
-	} 
+	}
 	assert(Node->memflag == NODE_IN_USE);
 
 	// update existing flow
 	Node->packets++;
-	Node->bytes += NewNode->bytes; 
-	Node->t_last = NewNode->t_last; 
-	TouchUDPNode(Node);
-
+	Node->bytes += NewNode->bytes;
+	Node->t_last = NewNode->t_last;
 	dbg_printf("Existing UDP flow: Packets: %u, Bytes: %u\n", Node->packets, Node->bytes);
 
 	Free_Node(NewNode);
@@ -343,7 +336,7 @@ struct FlowNode *Node;
 
 static inline void ProcessICMPFlow(FlowSource_t	*fs, struct FlowNode *NewNode ) {
 
-	// Flush ICMP directly 
+	// Flush ICMP directly
 	StorePcapFlow(fs, NewNode);
 	dbg_printf("Flush ICMP flow: Packets: %u, Bytes: %u\n", NewNode->packets, NewNode->bytes);
 
@@ -375,9 +368,8 @@ void ProcessFlowNode(FlowSource_t *fs, struct FlowNode *node) {
 		case IPPROTO_ICMPV6:
 			ProcessICMPFlow(fs, node);
 			break;
-		default: 
+		default:
 			ProcessOtherFlow(fs, node);
-
 	}
 
 } // End of ProcessFlowNode
@@ -386,12 +378,10 @@ void ProcessPacket(NodeList_t *NodeList, pcap_dev_t *pcap_dev, const struct pcap
 struct FlowNode	*Node;
 struct ip 	  *ip;
 void		  *payload, *defragmented;
-uint32_t	  size_ip, offset, data_len, payload_len;
+uint32_t	  size_ip, offset, data_len, payload_len, bytes;
 uint16_t	  version, ethertype, proto;
-#ifdef DEVEL
 char		  s1[64];
 char		  s2[64];
-#endif
 static unsigned pkg_cnt = 0;
 
 	pkg_cnt++;
@@ -399,6 +389,7 @@ static unsigned pkg_cnt = 0;
 
 	pcap_dev->proc_stat.packets++;
 	offset = pcap_dev->linkoffset;
+	defragmented = NULL;
 
 	Node = New_Node();
 	if ( !Node ) {
@@ -420,10 +411,11 @@ static unsigned pkg_cnt = 0;
 				case 0x800:	 // IPv4
 				case 0x86DD: // IPv6
 					break;
-				case 0x8100: {	// VLAN 
+				case 0x8100: {	// VLAN
 					do {
 						vlan_hdr_t *vlan_hdr = (vlan_hdr_t *)(data + offset);  // offset points to end of link layer
-						dbg_printf("VLAN ID: %u, type: 0x%x\n", ntohs(vlan_hdr->vlan_id), ntohs(vlan_hdr->type) );
+						dbg_printf("VLAN ID: %u, type: 0x%x\n",
+							ntohs(vlan_hdr->vlan_id), ntohs(vlan_hdr->type) );
 						ethertype = ntohs(vlan_hdr->type);
 /*
 pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
@@ -436,42 +428,38 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 					// redo ethertype evaluation
 					goto REDO_LINK;
 					} break;
-				case 0x806:	 // skip ARP
-					// silently skip ARP
-					pcap_dev->proc_stat.skipped++;
-					return;
-					break;
 				case 0x26:	 // ?? multicast router termination ??
+				case 0x32:	
+				case 0x806:	 // skip ARP
 				case 0x4305: // B.A.T.M.A.N. BATADV
 				case 0x886f: // MS NLB heartbeat
 				case 0x88a2: // ATA over ethernet
 				case 0x88cc: // CISCO LLDP
 				case 0x9000: // Loop
+				case 0x9003: 
 				case 0x880b: // PPP - rfc 7042
+				case 0x6558: // Ethernet Bridge
 					pcap_dev->proc_stat.skipped++;
-					if ( Node->proto ) {
-						// if it's an encap which we do not understand yet - push tunnel
-						Push_Node(NodeList, Node);
-					} else {
-						pcap_dev->proc_stat.skipped++;
-						dbg_printf("Skip Ethertype 0x%x", ethertype);
-						Free_Node(Node);
-					}
-					return;
+					Free_Node(Node);
+					goto END_FUNC;
 					break;
 				default:
 					pcap_dev->proc_stat.unknown++;
-					LogError("Unsupported link type: 0x%x, packet: %u", ethertype, pkg_cnt);
+					LogInfo("Unsupported ether type: 0x%x, packet: %u", ethertype, pkg_cnt);
 					Free_Node(Node);
-					return;
+					goto END_FUNC;
 			}
+	} else {
+		LogInfo("Unsupported link type: 0x%x, packet: %u", pcap_dev->linktype, pkg_cnt);
+		Free_Node(Node);
+		return;
 	}
 
 	if (hdr->caplen < offset) {
 		pcap_dev->proc_stat.short_snap++;
-		LogError("Short packet: %u/%u", hdr->caplen, offset);
+		LogInfo("Short packet: %u/%u", hdr->caplen, offset);
 		Free_Node(Node);
-		return;
+		goto END_FUNC;
 	}
 
 	Node->t_first.tv_sec = hdr->ts.tv_sec;
@@ -482,7 +470,6 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 	data	 = data + offset;
 	data_len = hdr->caplen - offset;
 	offset	 = 0;
-	defragmented = NULL;
 
 	// IP decoding
 	REDO_IPPROTO:
@@ -491,6 +478,7 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 		// data is sitting on a defragmented IPv4 packet memory region
 		// REDO loop could result in a memory leak, if again IP is fragmented
 		// XXX memory leak to be fixed
+		LogError("Fragmentation memory leak triggered!");
 	}
 
 	ip  	= (struct ip *)(data + offset); // offset points to end of link layer
@@ -503,23 +491,20 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 		offset = size_ip;	// offset point to end of IP header
 
 		if ( data_len < size_ip ) {
-			LogError("Packet: %u Length error: data_len: %u < size IPV6: %u, captured: %u, hdr len: %u", 
+			LogInfo("Packet: %u Length error: data_len: %u < size IPV6: %u, captured: %u, hdr len: %u",
 				pkg_cnt, data_len, size_ip, hdr->caplen, hdr->len);	
 			pcap_dev->proc_stat.short_snap++;
 			Free_Node(Node);
-			return;
+			goto END_FUNC;
 		}
 
 		// XXX Extension headers not processed
 		proto		= ip6->ip6_ctlun.ip6_un1.ip6_un1_nxt;
-		payload_len = ntohs(ip6->ip6_ctlun.ip6_un1.ip6_un1_plen);
+		payload_len = bytes = ntohs(ip6->ip6_ctlun.ip6_un1.ip6_un1_plen);
 
 		if (data_len < (payload_len + size_ip) ) {
-			LogError("Packet: %u Length error: data_len: %u < payload_len %u + size IPv6", 
-				pkg_cnt, data_len, payload_len);
-			pcap_dev->proc_stat.short_snap++;
-			Free_Node(Node);
-			return;
+			// capture len was limited - so adapt payload_len
+			payload_len = data_len - size_ip;
 		}
 
 		dbg_printf("Packet IPv6, SRC %s, DST %s, ",
@@ -544,24 +529,23 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 		offset = size_ip;	// offset point to end of IP header
 
 		if ( data_len < size_ip ) {
-			LogError("Packet: %u Length error: data_len: %u < size IPV4: %u, captured: %u, hdr len: %u", 
+			LogInfo("Packet: %u Length error: data_len: %u < size IPV4: %u, captured: %u, hdr len: %u",
 				pkg_cnt, data_len, size_ip, hdr->caplen, hdr->len);	
 			pcap_dev->proc_stat.short_snap++;
 			Free_Node(Node);
-			return;
+			goto END_FUNC;
 		}
 
 		payload_len = ntohs(ip->ip_len);
-		dbg_printf("size IP hader: %u, len: %u, %u\n", size_ip, ip->ip_len, payload_len);
+		dbg_printf("size IP hader: %u, len: %u\n", size_ip, payload_len);
 
 		payload_len -= size_ip;	// ajust length compatibel IPv6
 		payload = (void *)ip + size_ip;
 		proto   = ip->ip_p;
 
-
 		if (data_len < (payload_len + size_ip) ) {
-			LogError("Packet: %u Length error: data_len: %u < payload_len %u + size IPv4, captured: %u, hdr len: %u", 
-				pkg_cnt, data_len, payload_len, hdr->caplen, hdr->len);
+			// capture len was limited - so adapt payload_len
+			payload_len = data_len - size_ip;
 			pcap_dev->proc_stat.short_snap++;
 		}
 
@@ -570,16 +554,33 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 			inet_ntop(AF_INET, &ip->ip_dst, s2, sizeof(s2)));
 
 		// IPv4 defragmentation
-		if ( (ip_off & IP_MF) || frag_offset ) { 
+		if ( (ip_off & IP_MF) || frag_offset ) {
 			uint16_t ip_id = ntohs(ip->ip_id);
+#ifdef DEVEL
+			if ( frag_offset == 0 )
+				printf("Fragmented packet: first segement: ip_off: %u, frag_offset: %u\n",
+					ip_off, frag_offset);
+			if (( ip_off & IP_MF ) && frag_offset )
+				printf("Fragmented packet: middle segement: ip_off: %u, frag_offset: %u\n",
+					ip_off, frag_offset);
+			if (( ip_off & IP_MF ) == 0  )
+				printf("Fragmented packet: last segement: ip_off: %u, frag_offset: %u\n",
+					ip_off, frag_offset);
+#endif
 			// fragmented packet
-			defragmented = IPFrag_tree_Update(ip->ip_src.s_addr, ip->ip_dst.s_addr, ip_id, &payload_len, ip_off, payload);
-			if ( defragmented == NULL ) 
+			defragmented = IPFrag_tree_Update(hdr->ts.tv_sec, ip->ip_src.s_addr, ip->ip_dst.s_addr,
+				ip_id, &payload_len, ip_off, payload);
+			if ( defragmented == NULL ) {
 				// not yet complete
-				return;
+				dbg_printf("Fragmentation not yet completed. Size %u bytes\n", payload_len);
+				Free_Node(Node);
+				goto END_FUNC;
+			}
+			dbg_printf("Fragmentation complete\n");
 			// packet defragmented - set payload to defragmented data
 			payload = defragmented;
-		} 
+		}
+		bytes 		= payload_len;
 
 		Node->src_addr.v6[0] = 0;
 		Node->src_addr.v6[1] = 0;
@@ -590,16 +591,16 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 		Node->dst_addr.v4 = ntohl(ip->ip_dst.s_addr);
 		Node->version = AF_INET;
 	} else {
-		LogError("ProcessPacket() Unsupprted protocol version: %i", version);
+		LogInfo("ProcessPacket() Unsupported protocol version: %i", version);
 		pcap_dev->proc_stat.unknown++;
 		Free_Node(Node);
-		return;
+		goto END_FUNC;
 	}
 
 	Node->packets = 1;
-	Node->bytes   = payload_len;
+	Node->bytes   = bytes;
 	Node->proto   = proto;
-	dbg_printf("Size: %u\n", payload_len);
+	dbg_printf("Payload: %u bytes, Full packet: %u bytes\n", payload_len, bytes);
 
 	// TCP/UDP decoding
 	switch (proto) {
@@ -607,43 +608,38 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 			struct udphdr *udp = (struct udphdr *)payload;
 			uint16_t UDPlen = ntohs(udp->uh_ulen);
 			if ( UDPlen < 8 ) {
-				LogError("UDP payload legth error: %u bytes < 8\n",
-					UDPlen);
-				if ( defragmented ) {
-					free(defragmented);
-					defragmented = NULL;
-				}
+				LogInfo("UDP payload length error: %u bytes < 8, SRC %s, DST %s",
+					UDPlen, inet_ntop(AF_INET, &ip->ip_src, s1, sizeof(s1)),
+					inet_ntop(AF_INET, &ip->ip_dst, s2, sizeof(s2)));
+
 				Free_Node(Node);
-				return;
+				break;
 			}
 			uint32_t size_udp_payload = ntohs(udp->uh_ulen) - 8;
 
-			if ( (payload_len - sizeof(struct udphdr)) != size_udp_payload ) {
-				LogError("UDP payload legth error: Expected %u, have %u bytes\n",
-					size_udp_payload, (payload_len - (unsigned)sizeof(struct udphdr)));
-
-				if ( defragmented ) {
-					free(defragmented);
-					defragmented = NULL;
-				}
+			if ( (bytes == payload_len ) && (payload_len - sizeof(struct udphdr)) < size_udp_payload ) {
+				LogInfo("UDP payload length error: Expected %u, have %u bytes, SRC %s, DST %s",
+					size_udp_payload, (payload_len - (unsigned)sizeof(struct udphdr)),
+					inet_ntop(AF_INET, &ip->ip_src, s1, sizeof(s1)),
+					inet_ntop(AF_INET, &ip->ip_dst, s2, sizeof(s2)));
 				Free_Node(Node);
-				return;
+				break;
 			}
 			payload = payload + sizeof(struct udphdr);
 			payload_len -= sizeof(struct udphdr);
-			dbg_printf("UDP: size: %u, SRC: %i, DST: %i\n", 
+			dbg_printf("UDP: size: %u, SRC: %i, DST: %i\n",
 				size_udp_payload, ntohs(udp->uh_sport), ntohs(udp->uh_dport));
 
+			Node->bytes = payload_len;
 			Node->flags = 0;
 			Node->src_port = ntohs(udp->uh_sport);
 			Node->dst_port = ntohs(udp->uh_dport);
 
 			if ( hdr->caplen == hdr->len ) {
 				// process payload of full packets
-				if ( Node->src_port == 53 || Node->dst_port == 53 ) 
+				if ( (bytes == payload_len) && (Node->src_port == 53 || Node->dst_port == 53) )
  					content_decode_dns(Node, payload, payload_len);
 			}
-
 			Push_Node(NodeList, Node);
 			} break;
 		case IPPROTO_TCP: {
@@ -652,21 +648,21 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 			size_tcp = tcp->th_off << 2;
 
 			if ( payload_len < size_tcp ) {
-				LogError("TCP header length error: len: %u < size TCP header: %u", payload_len, size_tcp);	
+				LogInfo("TCP header length error: len: %u < size TCP header: %u, SRC %s, DST %s",
+					payload_len, size_tcp,
+					inet_ntop(AF_INET, &ip->ip_src, s1, sizeof(s1)),
+					inet_ntop(AF_INET, &ip->ip_dst, s2, sizeof(s2)));
+                pcap_dev->proc_stat.short_snap++;
+
 				pcap_dev->proc_stat.short_snap++;
-				if ( defragmented ) {
-					free(defragmented);
-					defragmented = NULL;
-				}
 				Free_Node(Node);
-				return;
+				break;
 			}
 
 			payload = payload + size_tcp;
 			payload_len -= size_tcp;
 			dbg_printf("Size TCP header: %u, size TCP payload: %u ", size_tcp, payload_len);
-			// XXX Debug stuff - remove when released ...
-			dbg_printf("src %i, DST %i, flags %i : ", 
+			dbg_printf("src %i, DST %i, flags %i : ",
 				ntohs(tcp->th_sport), ntohs(tcp->th_dport), tcp->th_flags);
 
 #ifdef DEVEL
@@ -689,28 +685,28 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 			struct icmp *icmp = (struct icmp *)payload;
 
 			Node->dst_port = (icmp->icmp_type << 8 ) + icmp->icmp_code;
-			dbg_printf("IPv%d ICMP proto: %u, type: %u, code: %u\n", version, ip->ip_p, icmp->icmp_type, icmp->icmp_code);
+			dbg_printf("IPv%d ICMP proto: %u, type: %u, code: %u\n",
+				version, ip->ip_p, icmp->icmp_type, icmp->icmp_code);
+			Node->bytes -= sizeof(struct udphdr);
 			Push_Node(NodeList, Node);
 			} break;
 		case IPPROTO_ICMPV6: {
 			struct icmp6_hdr *icmp6 = (struct icmp6_hdr *)payload;
 
 			Node->dst_port = (icmp6->icmp6_type << 8 ) + icmp6->icmp6_code;
-			dbg_printf("IPv%d ICMP proto: %u, type: %u, code: %u\n", version, ip->ip_p, icmp6->icmp6_type, icmp6->icmp6_code);
+			dbg_printf("IPv%d ICMP proto: %u, type: %u, code: %u\n",
+				version, ip->ip_p, icmp6->icmp6_type, icmp6->icmp6_code);
 			Push_Node(NodeList, Node);
 			} break;
 		case IPPROTO_IPV6: {
 			uint32_t size_inner_ip = sizeof(struct ip6_hdr);
 
 			if ( payload_len < size_inner_ip ) {
-				LogError("IPIPv6 tunnel header length error: len: %u < size inner IP: %u", payload_len, size_inner_ip);	
+				LogInfo("IPIPv6 tunnel header length error: len: %u < size inner IP: %u",
+					payload_len, size_inner_ip);	
 				pcap_dev->proc_stat.short_snap++;
-				if ( defragmented ) {
-					free(defragmented);
-					defragmented = NULL;
-				}
 				Free_Node(Node);
-				return;
+				goto END_FUNC;
 			}
 			offset   = 0;
 			data 	 = payload;
@@ -731,14 +727,11 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 			uint32_t size_inner_ip = (inner_ip->ip_hl << 2);
 
 			if ( payload_len < size_inner_ip ) {
-				LogError("IPIP tunnel header length error: len: %u < size inner IP: %u", payload_len, size_inner_ip);	
+				LogInfo("IPIP tunnel header length error: len: %u < size inner IP: %u",
+					payload_len, size_inner_ip);	
 				pcap_dev->proc_stat.short_snap++;
-				if ( defragmented ) {
-					free(defragmented);
-					defragmented = NULL;
-				}
 				Free_Node(Node);
-				return;
+				break;
 			}
 			offset   = 0;
 			data 	 = payload;
@@ -760,14 +753,11 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 			uint32_t gre_hdr_size = sizeof(gre_hdr_t); // offset points to end of inner IP
 
 			if ( payload_len < gre_hdr_size ) {
-				LogError("GRE tunnel header length error: len: %u < size GRE hdr: %u", payload_len, gre_hdr_size);	
+				LogError("GRE tunnel header length error: len: %u < size GRE hdr: %u",
+					payload_len, gre_hdr_size);	
 				pcap_dev->proc_stat.short_snap++;
-				if ( defragmented ) {
-					free(defragmented);
-					defragmented = NULL;
-				}
 				Free_Node(Node);
-				return;
+				break;
 			}
 
 			dbg_printf("GRE proto encapsulation: type: 0x%x\n", ethertype);
@@ -785,15 +775,19 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 			goto REDO_LINK;
 
 			} break;
-		default: 
-			// not handled protocol - simply save node
-			Push_Node(NodeList, Node);
+		default:
+			// not handled protocol
+			Free_Node(Node);
 			pcap_dev->proc_stat.unknown++;
 			break;
 	}
 
+	END_FUNC:
+	if ( defragmented ) {
+		free(defragmented);
+		defragmented = NULL;
+		dbg_printf("Defragmented buffer freed for proto %u", proto);	
+	}
+
 
 } // End of ProcessPacket
-
-
-
