@@ -1,7 +1,5 @@
 /*
- *  Copyright (c) 2016, Peter Haag
- *  Copyright (c) 2014, Peter Haag
- *  Copyright (c) 2009, Peter Haag
+ *  Copyright (c) 2009-2020, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *  
@@ -29,11 +27,11 @@
  *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
  *  POSSIBILITY OF SUCH DAMAGE.
  *  
- *  Author: peter
- *
  */
 
+#ifdef HAVE_CONFIG_H 
 #include "config.h"
+#endif
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -55,7 +53,6 @@
 #include <time.h>
 #include <fcntl.h>
 #include <signal.h>
-#include <syslog.h>
 #include <sys/mman.h>
 #include <string.h>
 #include <dirent.h>
@@ -69,13 +66,13 @@
 #endif
 
 #include "util.h"
+#include "nfdump.h"
 #include "nffile.h"
 #include "nfx.h"
-#include "nf_common.h"
 #include "nfnet.h"
 #include "bookkeeper.h"
-#include "nfxstat.h"
 #include "collector.h"
+#include "launch.h"
 #include "flist.h"
 #include "nfstatfile.h"
 
@@ -92,12 +89,12 @@
 
 #include "expire.h"
 
-#include "sflow.h"
+#include "sflow_nfdump.h"
 
 #define DEFAULTSFLOWPORT "6343"
 
 /* Global Variables */
-caddr_t		shmem;
+void *shmem;
 
 /* globals */
 int verbose = 0;
@@ -125,8 +122,8 @@ static void daemonize(void);
 
 static void SetPriv(char *userid, char *groupid );
 
-static void run(packet_function_t receive_packet, int socket, send_peer_t peer, 
-	time_t twin, time_t t_begin, int report_seq, int use_subdirs, char *time_extension, int compress, int do_xstat);
+static void run(packet_function_t receive_packet, int socket, repeater_t *repeater,
+	time_t twin, time_t t_begin, int report_seq, int use_subdirs, char *time_extension, int compress);
 
 /* Functions */
 static void usage(char *name) {
@@ -145,10 +142,11 @@ static void usage(char *name) {
 					"-H Add port histogram data to flow file.(default 'no')\n"
 					"-n Ident,IP,logdir\tAdd this flow source - multiple streams\n" 
 					"-P pidfile\tset the PID file\n"
-					"-R IP[/port]\tRepeat incoming packets to IP address/port\n"
+					"-R IP[/port]\tRepeat incoming packets to IP address/port. Max 8 repeaters\n"
 					"-x process\tlaunch process after a new file becomes available\n"
-					"-j\t\tBZ2 compress flows in output file.\n"
 					"-z\t\tLZO compress flows in output file.\n"
+					"-y\t\tLZ4 compress flows in output file.\n"
+					"-j\t\tBZ2 compress flows in output file.\n"
 					"-B bufflen\tSet socket buffer to bufflen bytes\n"
 					"-e\t\tExpire data at each cycle.\n"
 					"-D\t\tFork to background\n"
@@ -168,7 +166,7 @@ pid_t ret;
 		return;
 
 	if ( launcher_alive ) {
-		syslog(LOG_INFO, "Signal launcher[%i] to terminate.", pid);
+		LogInfo("Signal launcher[%i] to terminate.", pid);
 		kill(pid, SIGTERM);
 
 		// wait for launcher to teminate
@@ -178,22 +176,22 @@ pid_t ret;
 			sleep(1);
 		}
 		if ( i >= LAUNCHER_TIMEOUT ) {
-			syslog(LOG_WARNING, "Laucher does not want to terminate - signal again");
+			LogError("Launcher does not want to terminate - signal again");
 			kill(pid, SIGTERM);
 			sleep(1);
 		}
 	} else {
-		syslog(LOG_ERR, "launcher[%i] already dead.", pid);
+		LogError("launcher[%i] already dead.", pid);
 	}
 
 	if ( (ret = waitpid (pid, &stat, 0)) == -1 ) {
-		syslog(LOG_ERR, "wait for launcher failed: %s %i", strerror(errno), ret);
+		LogError("wait for launcher failed: %s %i", strerror(errno), ret);
 	} else {
 		if ( WIFEXITED(stat) ) {
-			syslog(LOG_INFO, "launcher exit status: %i", WEXITSTATUS(stat));
+			LogInfo("launcher exit status: %i", WEXITSTATUS(stat));
 		}
 		if (  WIFSIGNALED(stat) ) {
-			syslog(LOG_WARNING, "launcher terminated due to signal %i", WTERMSIG(stat));
+			LogError("launcher terminated due to signal %i", WTERMSIG(stat));
 		}
 	}
 
@@ -285,7 +283,7 @@ int		err;
 	newuid = newgid = 0;
 	myuid = getuid();
 	if ( myuid != 0 ) {
-		syslog(LOG_ERR, "Only root wants to change uid/gid");
+		LogError("Only root wants to change uid/gid");
 		fprintf(stderr, "ERROR: Only root wants to change uid/gid\n");
 		exit(255);
 	}
@@ -311,7 +309,7 @@ int		err;
 
 		err = setgid(newgid);
 		if ( err ) {
-			syslog(LOG_ERR, "Can't set group id %ld for group '%s': %s",   (long)newgid, groupid, strerror(errno));
+			LogError("Can't set group id %ld for group '%s': %s",   (long)newgid, groupid, strerror(errno));
 			fprintf (stderr,"Can't set group id %ld for group '%s': %s\n", (long)newgid, groupid, strerror(errno));
 			exit(255);
 		}
@@ -321,7 +319,7 @@ int		err;
 	if ( newuid ) {
 		err = setuid(newuid);
 		if ( err ) {
-			syslog(LOG_ERR, "Can't set user id %ld for user '%s': %s",   (long)newuid, userid, strerror(errno));
+			LogError("Can't set user id %ld for user '%s': %s",   (long)newuid, userid, strerror(errno));
 			fprintf (stderr,"Can't set user id %ld for user '%s': %s\n", (long)newuid, userid, strerror(errno));
 			exit(255);
 		}
@@ -329,11 +327,24 @@ int		err;
 
 } // End of SetPriv
 
+static void format_file_block_header(data_block_header_t *header) {
+	
+	printf("\n"
+"File Block Header: \n"
+"  NumBlocks     =  %10u\n"
+"  Size          =  %10u\n"
+"  id         	 =  %10u\n",
+		header->NumRecords,
+		header->size,
+		header->id);
+
+} // End of format_file_block_header
+
 #include "nffile_inline.c"
 #include "collector_inline.c"
 
-static void run(packet_function_t receive_packet, int socket, send_peer_t peer, 
-	time_t twin, time_t t_begin, int report_seq, int use_subdirs, char *time_extension, int compress, int do_xstat) {
+static void run(packet_function_t receive_packet, int socket, repeater_t *repeater,
+	time_t twin, time_t t_begin, int report_seq, int use_subdirs, char *time_extension, int compress) {
 FlowSource_t			*fs;
 struct sockaddr_storage sf_sender;
 socklen_t 	sf_sender_size = sizeof(sf_sender);
@@ -343,20 +354,18 @@ uint32_t	blast_cnt, blast_failures, ignored_packets;
 ssize_t		cnt;
 void 		*in_buff;
 int 		err;
-char 		*string;
 srecord_t	*commbuff;
 
-	Init_sflow();
+	Init_sflow(verbose);
 
 	in_buff  = malloc(NETWORK_INPUT_BUFF_SIZE);
 	if ( !in_buff ) {
-		syslog(LOG_ERR, "malloc() buffer allocation error: %s", strerror(errno));
+		LogError("malloc() buffer allocation error: %s", strerror(errno));
 		return;
 	}
 
 	// init vars
 	commbuff = (srecord_t *)shmem;
-	string = NULL;
 
 	// Init each netflow source output data buffer
 	fs = FlowSource;
@@ -367,11 +376,6 @@ srecord_t	*commbuff;
 		if ( !fs->nffile ) {
 			return;
 		}
-		if ( do_xstat ) {
-			fs->xstat = InitXStat(fs->nffile);
-			if ( !fs->xstat ) 
-				return;
-		}
 
 		// init stat vars
 		fs->bad_packets		= 0;
@@ -388,8 +392,8 @@ srecord_t	*commbuff;
 	cnt = 0;
 	ignored_packets  = 0;
 
-	// wake up at least at next time slot (twin) + some Overdue time
-	alarm(t_start + twin + OVERDUE_TIME - time(NULL));
+	// wake up at least at next time slot (twin) + 1s
+	alarm(t_start + twin + 1 - time(NULL));
 	/*
 	 * Main processing loop:
 	 * this loop, continues until done = 1, set by the signal handler
@@ -398,6 +402,7 @@ srecord_t	*commbuff;
 	 */
 	while ( 1 ) {
 		struct timeval tv;
+		int i;
 
 		/* read next bunch of data into beginn of input buffer */
 		if ( !done) {
@@ -416,16 +421,19 @@ srecord_t	*commbuff;
 				(struct sockaddr *)&sf_sender, &sf_sender_size);
 #endif
 			if ( cnt == -1 && errno != EINTR ) {
-				syslog(LOG_ERR, "ERROR: recvfrom: %s", strerror(errno));
+				LogError("ERROR: recvfrom: %s", strerror(errno));
 				continue;
 			}
 
-			if ( peer.hostname ) {
+			i = 0;
+			while ( repeater[i].hostname && (i < MAX_REPEATERS)) {
 				ssize_t len;
-				len = sendto(peer.sockfd, in_buff, cnt, 0, (struct sockaddr *)&(peer.addr), peer.addrlen);
+				len = sendto(repeater[i].sockfd, in_buff, cnt, 0, 
+						(struct sockaddr *)&(repeater[i].addr), repeater[i].addrlen);
 				if ( len < 0 ) {
-					syslog(LOG_ERR, "ERROR: sendto(): %s", strerror(errno));
+					LogError("ERROR: sendto(): %s", strerror(errno));
 				}
+				i++;
 			}
 		}
 
@@ -434,31 +442,26 @@ srecord_t	*commbuff;
 		t_now = tv.tv_sec;
 
 		if ( ((t_now - t_start) >= twin) || done ) {
-			char subfilename[64];
 			struct  tm *now;
-			char	*subdir, fmt[64];
+			char	*subdir, fmt[MAXTIMESTRING];
+
 			alarm(0);
 			now = localtime(&t_start);
-			strftime(fmt, sizeof fmt, time_extension, now);
+			strftime(fmt, sizeof(fmt), time_extension, now);
 
 			// prepare sub dir hierarchy
 			if ( use_subdirs ) {
 				subdir = GetSubDir(now);
 				if ( !subdir ) {
 					// failed to generate subdir path - put flows into base directory
-					syslog(LOG_ERR, "Failed to create subdir path!");
+					LogError("Failed to create subdir path!");
 			
 					// failed to generate subdir path - put flows into base directory
 					subdir = NULL;
-					snprintf(subfilename, 63, "nfcapd.%s", fmt);
-				} else {
-					snprintf(subfilename, 63, "%s/nfcapd.%s", subdir, fmt);
 				}
 			} else {
 				subdir = NULL;
-				snprintf(subfilename, 63, "nfcapd.%s", fmt);
 			}
-			subfilename[63] = '\0';
 
 			// for each flow source update the stats, close the file and re-initialize the new file
 			fs = FlowSource;
@@ -469,52 +472,54 @@ srecord_t	*commbuff;
 
 				if ( verbose ) {
 					// Dump to stdout
-					format_file_block_header(nffile->block_header, &string, 0);
-					printf("%s\n", string);
+					format_file_block_header(nffile->block_header);
 				}
 
 				if ( nffile->block_header->NumRecords ) {
 					// flush current buffer to disc
 					if ( WriteBlock(nffile) <= 0 )
-						syslog(LOG_ERR, "Ident: %s, failed to write output buffer to disk: '%s'" , fs->Ident, strerror(errno));
+						LogError("Ident: %s, failed to write output buffer to disk: '%s'" ,
+							fs->Ident, strerror(errno));
 				} // else - no new records in current block
 
 	
 				// prepare filename
-				snprintf(nfcapd_filename, MAXPATHLEN-1, "%s/%s", fs->datadir, subfilename);
+				if ( subdir ) {
+					if ( SetupSubDir(fs->datadir, subdir, error, 255) ) {
+						snprintf(nfcapd_filename, MAXPATHLEN-1, "%s/%s/nfcapd.%s", fs->datadir, subdir, fmt);
+					} else {
+						LogError("Ident: %s, Failed to create sub hier directories: %s", fs->Ident, error );
+						// skip subdir - put flows directly into current directory
+						snprintf(nfcapd_filename, MAXPATHLEN-1, "%s/nfcapd.%s", fs->datadir, fmt);
+					}
+				} else {
+					snprintf(nfcapd_filename, MAXPATHLEN-1, "%s/nfcapd.%s", fs->datadir, fmt);
+				}
 				nfcapd_filename[MAXPATHLEN-1] = '\0';
-	
+
 				// update stat record
-				nffile->stat_record->first_seen 	= fs->first_seen/1000;
+				// if no flows were collected, fs->last_seen is still 0
+				// set first_seen to start of this time slot, with twin window size.
+				if ( fs->last_seen == 0 ) {
+					fs->first_seen = (uint64_t)1000 * (uint64_t)t_start;
+					fs->last_seen  = (uint64_t)1000 * (uint64_t)(t_start + twin);
+				}
+				nffile->stat_record->first_seen = fs->first_seen/1000;
 				nffile->stat_record->msec_first	= fs->first_seen - nffile->stat_record->first_seen*1000;
 				nffile->stat_record->last_seen 	= fs->last_seen/1000;
 				nffile->stat_record->msec_last	= fs->last_seen - nffile->stat_record->last_seen*1000;
 	
-				if ( fs->xstat ) {
-					if ( WriteExtraBlock(nffile, fs->xstat->block_header ) <= 0 ) 
-						syslog(LOG_ERR, "Ident: %s, failed to write xstat buffer to disk: '%s'" , fs->Ident, strerror(errno));
-
-					ResetPortHistogram(fs->xstat->port_histogram);
-					ResetBppHistogram(fs->xstat->bpp_histogram);
-				}
-
 				// Flush Exporter Stat to file
 				FlushExporterStats(fs);
 				// Write Stat record and close file
 				CloseUpdateFile(nffile, fs->Ident);
 
-				if ( subdir && !SetupSubDir(fs->datadir, subdir, error, 255) ) {
-					// in this case the flows get lost! - the rename will fail
-					// but this should not happen anyway, unless i/o problems, inode problems etc.
-					syslog(LOG_ERR, "Ident: %s, Failed to create sub hier directories: %s", fs->Ident, error );
-				}
-
 				// if rename fails, we are in big trouble, as we need to get rid of the old .current file
 				// otherwise, we will loose flows and can not continue collecting new flows
 				err = rename(fs->current, nfcapd_filename);
 				if ( err ) {
-					syslog(LOG_ERR, "Ident: %s, Can't rename dump file: %s", fs->Ident,  strerror(errno));
-					syslog(LOG_ERR, "Ident: %s, Serious Problem! Fix manually", fs->Ident);
+					LogError("Ident: %s, Can't rename dump file: %s", fs->Ident,  strerror(errno));
+					LogError("Ident: %s, Serious Problem! Fix manually", fs->Ident);
 					if ( launcher_pid )
 						commbuff->failed = 1;
 
@@ -531,8 +536,9 @@ srecord_t	*commbuff;
 				}
 
 				// log stats
-				syslog(LOG_INFO,"Ident: '%s' Flows: %llu, Packets: %llu, Bytes: %llu, Sequence Errors: %u, Bad Packets: %u", 
-					fs->Ident, (unsigned long long)nffile->stat_record->numflows, (unsigned long long)nffile->stat_record->numpackets, 
+				LogInfo("Ident: '%s' Flows: %llu, Packets: %llu, Bytes: %llu, Sequence Errors: %u, Bad Packets: %u", 
+					fs->Ident, (unsigned long long)nffile->stat_record->numflows,
+					(unsigned long long)nffile->stat_record->numpackets, 
 					(unsigned long long)nffile->stat_record->numbytes, nffile->stat_record->sequence_failure, fs->bad_packets);
 
 				// reset stat record
@@ -546,10 +552,6 @@ srecord_t	*commbuff;
 						LogError("killed due to fatal error: ident: %s", fs->Ident);
 						break;
 					}
-					/* XXX needs fixing */
-					if ( fs->xstat ) {
-						// Add catalog entry
-					}
 				}
 
 				// Dump all extension maps to the buffer
@@ -563,27 +565,26 @@ srecord_t	*commbuff;
 			if ( launcher_pid ) {
 				// Signal launcher
 		
-				// prepare filename for %f expansion
-				strncpy(commbuff->fname, subfilename, FNAME_SIZE-1);
-				commbuff->fname[FNAME_SIZE-1] = 0;
-				snprintf(commbuff->tstring, 16, "%i%02i%02i%02i%02i", 
-					now->tm_year + 1900, now->tm_mon + 1, now->tm_mday, now->tm_hour, now->tm_min);
-				commbuff->tstring[15] = 0;
+				strncpy(commbuff->tstring, fmt, MAXTIMESTRING);
+				commbuff->tstring[MAXTIMESTRING-1] = '\0';
+
 				commbuff->tstamp = t_start;
-				if ( subdir ) 
-					strncpy(commbuff->subdir, subdir, FNAME_SIZE);
-				else
-					commbuff->subdir[0] = '\0';
+				if ( subdir ) {
+					snprintf(commbuff->fname, MAXPATHLEN-1, "%s/nfcapd.%s", subdir, fmt);
+				} else {
+					snprintf(commbuff->fname, MAXPATHLEN-1, "nfcapd.%s", fmt);
+				}
+				commbuff->fname[MAXPATHLEN-1] = '\0';
 
 				if ( launcher_alive ) {
-					syslog(LOG_DEBUG, "Signal launcher");
+					LogInfo("Signal launcher");
 					kill(launcher_pid, SIGHUP);
 				} else 
-					syslog(LOG_ERR, "ERROR: Launcher did unexpectedly!");
+					LogError("ERROR: Launcher died unexpectedly!");
 
 			}
 			
-			syslog(LOG_INFO, "Total ignored packets: %u", ignored_packets);
+			LogInfo("Total ignored packets: %u", ignored_packets);
 			ignored_packets = 0;
 
 			if ( done )
@@ -593,10 +594,10 @@ srecord_t	*commbuff;
 			t_start += twin;
 			/* t_start = filename time stamp: begin of slot
 		 	* + twin = end of next time interval
-		 	* + OVERDUE_TIME = if no data is collected, this is at latest to act
+		 	* + 1 = if no data is collected, this is at latest to act
 		 	* - t_now = difference value to now
 		 	*/
-			alarm(t_start + twin + OVERDUE_TIME - t_now);
+			alarm(t_start + twin + 1 - t_now);
 		}
 
 		/* check for error condition or done . errno may only be EINTR */
@@ -611,7 +612,7 @@ srecord_t	*commbuff;
 				break;
 			else {
 				/* this should never be executed as it should be caught in other places */
-				syslog(LOG_ERR, "error condition in '%s', line '%d', cnt: %i", __FILE__, __LINE__ ,(int)cnt);
+				LogError("error condition in '%s', line '%d', cnt: %i", __FILE__, __LINE__ ,(int)cnt);
 				continue;
 			}
 		}
@@ -624,7 +625,7 @@ srecord_t	*commbuff;
 
 		fs = GetFlowSource(&sf_sender);
 		if ( fs == NULL ) {
-			syslog(LOG_WARNING, "Skip UDP packet. Ignored packets so far %u packets", ignored_packets);
+			LogError("Skip UDP packet. Ignored packets so far %u packets", ignored_packets);
 			ignored_packets++;
 			continue;
 		}
@@ -632,7 +633,7 @@ srecord_t	*commbuff;
 
 		/* check for too little data - cnt must be > 0 at this point */
 		if ( cnt < sizeof(common_flow_header_t) ) {
-			syslog(LOG_WARNING, "Ident: %s, Data length error: too little data for common netflow header. cnt: %i",fs->Ident, (int)cnt);
+			LogError("Ident: %s, Data length error: too little data for common netflow header. cnt: %i",fs->Ident, (int)cnt);
 			fs->bad_packets++;
 			continue;
 		}
@@ -651,7 +652,7 @@ srecord_t	*commbuff;
 			fs->nffile->block_header->size 		 = 0;
 			fs->nffile->block_header->NumRecords = 0;
 			fs->nffile->buff_ptr = (void *)((pointer_addr_t)fs->nffile->block_header + sizeof(data_block_header_t) );
-			syslog(LOG_ERR, "### Software bug ### Ident: %s, output buffer overflow: expect memory inconsitency", fs->Ident);
+			LogError("### Software bug ### Ident: %s, output buffer overflow: expect memory inconsitency", fs->Ident);
 		}
 	}
 
@@ -670,20 +671,22 @@ srecord_t	*commbuff;
 
 int main(int argc, char **argv) {
  
-char	*bindhost, *filter, *datadir, pidstr[32], *launch_process;
+char	*bindhost, *datadir, pidstr[32], *launch_process;
 char	*userid, *groupid, *checkptr, *listenport, *mcastgroup, *extension_tags;
-char	*Ident, *pcap_file, *time_extension, pidfile[MAXPATHLEN];
+char	*Ident, *time_extension, pidfile[MAXPATHLEN];
 struct stat fstat;
-srecord_t	*commbuff;
 packet_function_t receive_packet;
-send_peer_t  peer;
+repeater_t repeater[MAX_REPEATERS];
 FlowSource_t *fs;
 struct sigaction act;
 int		family, bufflen;
 time_t 	twin, t_start;
-int		sock, err, synctime, do_daemonize, expire, spec_time_extension, report_sequence, do_xstat;
+int		sock, synctime, do_daemonize, expire, spec_time_extension, report_sequence;
 int		subdir_index, compress;
-int	c;
+int		c, i;
+#ifdef PCAP
+char	*pcap_file = NULL;
+#endif
 
 	receive_packet 	= recvfrom;
 	verbose = synctime = do_daemonize = 0;
@@ -696,23 +699,22 @@ int	c;
 	bindhost 		= NULL;
 	mcastgroup		= NULL;
 	pidfile[0]		= 0;
-	filter   		= NULL;
 	launch_process	= NULL;
 	userid 			= groupid = NULL;
 	twin	 		= TIME_WINDOW;
 	datadir	 		= NULL;
 	subdir_index	= 0;
 	time_extension	= "%Y%m%d%H%M";
-	expire			= 0;
 	spec_time_extension = 0;
+	expire			= 0;
 	compress		= NOT_COMPRESSED;
-	do_xstat		= 0;
-	memset((void *)&peer, 0, sizeof(send_peer_t));
-	peer.family		= AF_UNSPEC;
+	memset((void *)&repeater, 0, sizeof(repeater));
+	for ( i = 0; i < MAX_REPEATERS; i++ ) {
+		repeater[i].family = AF_UNSPEC;
+	}
 	Ident			= "none";
 	FlowSource		= NULL;
 	extension_tags	= DefaultExtensions;
-	pcap_file		= NULL;
 
 	while ((c = getopt(argc, argv, "46ewhEVI:DB:b:f:jl:n:p:J:P:R:S:T:t:x:ru:g:zZ")) != EOF) {
 		switch (c) {
@@ -732,9 +734,6 @@ int	c;
 			case 'E':
 				verbose = 1;
 				break;
-			case 'H':
-				do_xstat = 1;
-				break;
 			case 'f': {
 #ifdef PCAP
 				struct stat	fstat;
@@ -767,14 +766,21 @@ int	c;
 				break;
 			case 'j':
 				if ( compress ) {
-					LogError("Use either -z for LZO or -j for BZ2 compression, but not both\n");
+					LogError("Use one compression: -z for LZO, -j for BZ2 or -y for LZ4 compression\n");
 					exit(255);
 				}
 				compress = BZ2_COMPRESSED;
 				break;
+			case 'y':
+				if ( compress ) {
+					LogError("Use one compression: -z for LZO, -j for BZ2 or -y for LZ4 compression\n");
+					exit(255);
+				}
+				compress = LZ4_COMPRESSED;
+				break;
 			case 'z':
 				if ( compress ) {
-					LogError("Use either -z for LZO or -j for BZ2 compression, but not both\n");
+					LogError("Use one compression: -z for LZO, -j for BZ2 or -y for LZ4 compression\n");
 					exit(255);
 				}
 				compress = LZO_COMPRESSED;
@@ -804,20 +810,34 @@ int	c;
 						exit(255);
 					}
 					tmp[MAXPATHLEN-1] = 0;
-					snprintf(pidfile, MAXPATHLEN - 1 - strlen(tmp), "%s/%s", tmp, optarg);
+					if ( (strlen(tmp) + strlen(optarg) + 3) < MAXPATHLEN ) {
+						snprintf(pidfile, MAXPATHLEN - 3 - strlen(tmp), "%s/%s", tmp, optarg);
+					} else {
+						fprintf(stderr, "pidfile MAXPATHLEN error:\n");
+						exit(255);
+					}
 				}
 				// pidfile now absolute path
 				pidfile[MAXPATHLEN-1] = 0;
 				break;
 			case 'R': {
+				char *port, *hostname;
 				char *p = strchr(optarg, '/');
+				int i = 0;
 				if ( p ) { 
 					*p++ = '\0';
-					peer.port = strdup(p);
+					port = strdup(p);
 				} else {
-					peer.port = DEFAULTSFLOWPORT;
+					port = DEFAULTSFLOWPORT;
 				}
-				peer.hostname = strdup(optarg);
+				hostname = strdup(optarg);
+				while ( repeater[i].hostname && (i < MAX_REPEATERS) ) i++;
+				if ( i == MAX_REPEATERS ) {
+					fprintf(stderr, "Too many packet repeaters! Max: %i repeaters allowed.\n", MAX_REPEATERS);
+					exit(255);
+				}
+				repeater[i].hostname = hostname;
+				repeater[i].port 	 = port;
 
 				break; }
 			case 'r':
@@ -829,7 +849,7 @@ int	c;
 					fprintf(stderr, "ERROR: Path too long!\n");
 					exit(255);
 				}
-				err  = stat(datadir, &fstat);
+				stat(datadir, &fstat);
 				if ( !(fstat.st_mode & S_IFDIR) ) {
 					fprintf(stderr, "No such directory: %s\n", datadir);
 					break;
@@ -848,12 +868,12 @@ int	c;
 				break;
 			case 't':
 				twin = atoi(optarg);
-				if ( twin <= 0 ) {
-					fprintf(stderr, "ERROR: time frame <= 0\n");
+				if ( twin < 2 ) {
+					LogError("time interval <= 2s not allowed");
 					exit(255);
 				}
 				if (twin < 60) {
-					fprintf(stderr, "WARNING, Very small time frame - < 60s!\n");
+					time_extension	= "%Y%m%d%H%M%S";
 				}
 				break;
 			case 'x':
@@ -906,7 +926,7 @@ int	c;
 		exit(255);
 	}
 
-	if ( !InitLog(argv[0], SYSLOG_FACILITY)) {
+	if ( !InitLog(do_daemonize, argv[0], SYSLOG_FACILITY, verbose) ) {
 		exit(255);
 	}
 
@@ -929,11 +949,14 @@ int	c;
 		exit(255);
 	}
 
-	if ( peer.hostname ) {
-		peer.sockfd = Unicast_send_socket (peer.hostname, peer.port, peer.family, bufflen, 
-											&peer.addr, &peer.addrlen );
-		if ( peer.sockfd <= 0 )
+	i = 0;
+	while ( repeater[i].hostname && (i < MAX_REPEATERS) ) {
+		repeater[i].sockfd = Unicast_send_socket (repeater[i].hostname, repeater[i].port, repeater[i].family, bufflen, 
+											&repeater[i].addr, &repeater[i].addrlen );
+		if ( repeater[i].sockfd <= 0 )
 			exit(255);
+		LogInfo("Replay flows to host: %s port: %s", repeater[i].hostname, repeater[i].port);
+		i++;
 	}
 
 	SetPriv(userid, groupid);
@@ -988,7 +1011,7 @@ int	c;
 		exit(255);
 	} else {
 		/* user specified a pcap filter */
-		filter = argv[optind];
+		// not used: filter = argv[optind];
 	}
 
 	t_start = time(NULL);
@@ -1003,7 +1026,7 @@ int	c;
 		pid_t pid = getpid();
 		int pidf  = open(pidfile, O_RDWR|O_TRUNC|O_CREAT, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
 		if ( pidf == -1 ) {
-			syslog(LOG_ERR, "Error opening pid file: '%s' %s", pidfile, strerror(errno));
+			LogError("Error opening pid file: '%s' %s", pidfile, strerror(errno));
 			close(sock);
 			exit(255);
 		}
@@ -1020,24 +1043,22 @@ int	c;
 		// as well as shared memory
 		// prepare shared memory
 		shmem = mmap(0, sizeof(srecord_t), PROT_READ | PROT_WRITE, MAP_ANON | MAP_SHARED, -1, 0);
-		if ( shmem == (caddr_t)-1 ) {
-			syslog(LOG_ERR, "mmap() error: %s", strerror(errno));
+		if ( shmem == MAP_FAILED ) {
+			LogError("mmap() error: %s", strerror(errno));
 			close(sock);
 			exit(255);
 		}
 
-		commbuff = (srecord_t *)shmem;
-
 		launcher_pid = fork();
 		switch (launcher_pid) {
 			case 0:
 				// child
 				close(sock);
-				launcher((char *)shmem, FlowSource, launch_process, expire);
+				launcher(shmem, FlowSource, launch_process, expire);
 				exit(0);
 				break;
 			case -1:
-				syslog(LOG_ERR, "fork() error: %s", strerror(errno));
+				LogError("fork() error: %s", strerror(errno));
 				if ( strlen(pidfile) )
 					unlink(pidfile);
 				exit(255);
@@ -1045,14 +1066,14 @@ int	c;
 			default:
 				// parent
 			launcher_alive = 1;
-			syslog(LOG_DEBUG, "Launcher[%i] forked", launcher_pid);
+			LogInfo("Launcher[%i] forked", launcher_pid);
 		}
 	}
 
 	fs = FlowSource;
 	while ( fs ) {
 		if ( InitBookkeeper(&fs->bookkeeper, fs->datadir, getpid(), launcher_pid) != BOOKKEEPER_OK ) {
-			syslog(LOG_ERR, "initialize bookkeeper failed.");
+			LogError("initialize bookkeeper failed.");
 
 			// release all already allocated bookkeepers
 			fs = FlowSource;
@@ -1070,7 +1091,6 @@ int	c;
 
 		// Init the extension map list
 		if ( !InitExtensionMapList(fs) ) {
-			// error message goes to syslog
 			exit(255);
 		}
 
@@ -1088,9 +1108,9 @@ int	c;
 	sigaction(SIGALRM, &act, NULL);
 	sigaction(SIGCHLD, &act, NULL);
 
-	syslog(LOG_INFO, "Startup.");
-	run(receive_packet, sock, peer, twin, t_start, report_sequence, subdir_index, 
-		time_extension, compress, do_xstat);
+	LogInfo("Startup.");
+	run(receive_packet, sock, repeater, twin, t_start, report_sequence, subdir_index, 
+		time_extension, compress);
 	close(sock);
 	kill_launcher(launcher_pid);
 
@@ -1101,15 +1121,15 @@ int	c;
 		if ( expire == 0 && ReadStatInfo(fs->datadir, &dirstat, LOCK_IF_EXISTS) == STATFILE_OK ) {
 			UpdateBookStat(dirstat, fs->bookkeeper);
 			WriteStatInfo(dirstat);
-			syslog(LOG_INFO, "Updating statinfo in directory '%s'", datadir);
+			LogInfo("Updating statinfo in directory '%s'", datadir);
 		}
 
 		ReleaseBookkeeper(fs->bookkeeper, DESTROY_BOOKKEEPER);
 		fs = fs->next;
 	}
 
-	syslog(LOG_INFO, "Terminating sfcapd.");
-	closelog();
+	LogInfo("Terminating sfcapd.");
+	EndLog();
 
 	if ( strlen(pidfile) )
 		unlink(pidfile);
