@@ -9,15 +9,18 @@ $DS{'v4_basic_txt'} = {
 	
 	'bytes' => '750000000',	# 12.5MB * 60s = (100mb/s)
 	'pkts'  => '1500000',
+	'packets'  => '1500000', # from 1.26
 
 	'srcport' => '53008',
 	'dstport' => '10050',
 	'tcpflags' => '27',
+	'flags' => '27', # from 1.26
 
 	'srcip' => '147.229.3.135',
 	'dstip' => '10.255.5.6',
 	'inetfamily' => 'ipv4',
 	'nexthop' => '10.255.5.1',
+	'nextip' => '10.255.5.1', # from 1.25
 
 	'proto' => '6',
 
@@ -40,18 +43,24 @@ $DS{'v4_txt'} = {
 	
 	'bytes' => '750000000',
 	'pkts' => '1500000',
+	'packets' => '1500000',  # from 1.26
 	'outbytes' => '291',
 	'outpkts' => '5',
+	'outpackets' => '5',  # from 1.26
 	'flows' => '1',
 
 	'srcport' => '53008',
 	'dstport' => '10050',
 	'tcpflags' => '27',
+	'flags' => '27', # from 1.26
 
 	'srcip' => '147.229.3.135',
+	'srcnet' => '147.229.3.135',
 	'dstip' => '10.255.5.6',
+	'dstnet' => '10.255.5.6',
 	'inetfamily' => 'ipv4',
 	'nexthop' => '10.255.5.1',
+	'nextip' => '10.255.5.1', # from 1.25
 	'srcmask' => '24',
 	'dstmask' => '32',
 	'tos' => '7',
@@ -80,8 +89,11 @@ $DS{'v4_txt'} = {
 	'fwd' => '1',
 
 	'router' => '10.255.5.6',
+	'routerip' => '10.255.5.6',
    	'sysid' => '0',
+   	'engine-id' => '0', # from 1.25
    	'systype' => '0',
+   	'engine-type' => '0', # from 1.25
 
 	'cl' => '100', 
 	'sl' => '200',
@@ -95,6 +107,7 @@ $DS{'v4_txt'} = {
 	'if' => '2',
 	'port' => '53008',
 	'ip' => '147.229.3.135',
+	'net' => '147.229.3.135', # from 1.25
 	'as' => '1234568',
 	'vlan' => '10',
 };
@@ -102,12 +115,17 @@ $DS{'v4_txt'} = {
 # prepare v6 structure - same as V4 but address changed to v6
 $DS{'v6_txt'} = { %{$DS{'v4_txt'}} };
 $DS{'v6_txt'}->{'srcip'} ='2001:67c:1220:f565::93e5:f0fb';
+$DS{'v6_txt'}->{'srcnet'} ='2001:67c:1220:f565::93e5:f0fb';
 $DS{'v6_txt'}->{'dstip'} ='2001:abc:1220:f565::93e5:f0fb';
+$DS{'v6_txt'}->{'dstnet'} ='2001:abc:1220:f565::93e5:f0fb';
 $DS{'v6_txt'}->{'inetfamily'} = 'ipv6';
 $DS{'v6_txt'}->{'nexthop'} ='2001:67c:1220:f565::1';
+$DS{'v6_txt'}->{'nextip'} ='2001:67c:1220:f565::1';
 $DS{'v6_txt'}->{'bgpnexthop'} ='2001:67c:1220:f565::1';
 $DS{'v6_txt'}->{'router'} ='2001:67c:1220:f565::10';
+$DS{'v6_txt'}->{'routerip'} ='2001:67c:1220:f565::10';
 $DS{'v6_txt'}->{'ip'} ='2001:67c:1220:f565::93e5:f0fb';
+$DS{'v6_txt'}->{'net'} ='2001:67c:1220:f565::93e5:f0fb';
 
 $DS{'v4_raw'} = txt2flow( $DS{'v4_txt'} );
 $DS{'v4_basic_raw'} = txt2flow( $DS{'v4_basic_txt'} );
